// Thread.h: interface for the CThread class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_THREAD_H__F73FAE82_F3F2_4497_9786_833B36A6FC4C__INCLUDED_)
#define AFX_THREAD_H__F73FAE82_F3F2_4497_9786_833B36A6FC4C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define WM_EXIT			WM_USER + 256
#define WM_EMPTY		WM_USER + 257




#ifndef MYASSERT
	#define MYASSERT(expr) _ASSERTE(expr)
#endif


#define BEGIN_THREAD_MAP()	void __MainLoop(){									\
							MSG msg;											\
							PeekMessage(&msg,NULL, 0,0,PM_NOREMOVE);			\
							SetEvent(m_hEventStarted);							\
							for (;;)	{										\
								if (0==GetMessage(&msg,NULL,0,0)) return;		\
								switch(msg.message){				

#define THREAD_HANDLER(M, F)		case M:	{									\
										ResetEvent(m_hIdle);						\
										BOOL succ = F(msg.wParam, msg.lParam);	\
										SetEvent(m_hIdle);					\
										if (!succ) return;} break;				

								

#define END_THREAD_MAP()			case WM_EXIT:								\
										OnEmptyQueue();							\
										PostQuitMessage(0);						\
										break;									\
									case WM_EMPTY:								\
										OnEmptyQueue();							\
										break;									\
								}												\
								if (WaitForSingleObject(m_hEmptyQueue,0)== WAIT_OBJECT_0)	OnEmptyQueue();		\
								ResetEvent(m_hEventAbort);														\
								}}			
	


class CThreadBase  
{
	HANDLE m_hMutex;
	
	
	
	

	

	int m_timeout;

protected:



	BOOL m_running;
	UINT m_id;

	HANDLE m_hEventAbort;
	HANDLE m_hEmptyQueue;
	HANDLE m_hIdle;
	HANDLE m_hEventFinished;

	HANDLE m_hEventStarted;

public:

	UINT GetThreadId() { return m_id; }

	//abort _current_ operation....
	void Abort()
	{
		SetEvent(m_hEventAbort);
	}

	BOOL IsIdle()
	{
		return WaitForIdle(0);
	}


	BOOL WaitForIdle(DWORD timeout)
	{
		return (WaitForSingleObject(m_hIdle,timeout) == WAIT_OBJECT_0);
	}

	//abort all outstanding operations (client thread)
	void EmptyQueue()	
	{
		MYASSERT(m_running);
		SetEvent(m_hEmptyQueue);
		PostMessage(WM_EMPTY);
		while (WaitForSingleObject(m_hEmptyQueue,0) == WAIT_OBJECT_0)
			Sleep(10);		
	}


	void Close()		//comes in on _other thread_
	{
		MYASSERT(m_running);
		PostMessage(WM_EXIT);
		if (WAIT_OBJECT_0 != WaitForSingleObject(m_hEventFinished,m_timeout))
			throw "Thread blocked....";
	}



	void PostMessage(UINT msg, WPARAM wParam = NULL, LPARAM lParam=NULL)
	{
		MYASSERT(m_running);
		if (!PostThreadMessage(m_id,msg,wParam,lParam))
		{
			Sleep(1000);
			if (!PostThreadMessage(m_id,msg,wParam,lParam))
			{
				DWORD err = GetLastError();
				MYASSERT(FALSE);
			}
		}
	}



	CThreadBase(int timeout = 1000) : m_timeout(timeout), m_running(FALSE)
	{
	
		m_hMutex			= CreateMutex(NULL,FALSE,NULL);
		m_hEventAbort		= CreateEvent(NULL,TRUE,FALSE,NULL);
		m_hEventStarted		= CreateEvent(NULL,TRUE,FALSE,NULL);
		m_hIdle				= CreateEvent(NULL,TRUE,TRUE,NULL);
		m_hEmptyQueue		= CreateEvent(NULL,TRUE,FALSE,NULL);
		m_hEventFinished	= CreateEvent(NULL,TRUE,FALSE,NULL);

		
	}

	virtual ~CThreadBase()
	{
		MYASSERT(!m_running);

		CloseHandle(m_hMutex);
		CloseHandle(m_hEventAbort);
		CloseHandle(m_hEmptyQueue);
		CloseHandle(m_hEventFinished);
		CloseHandle(m_hIdle);
		CloseHandle(m_hEventStarted);
	}

protected:

	//respond to empty queue request (this thread)
	void OnEmptyQueue()
	{
		MSG msg;
		BOOL ex;
		while (ex=PeekMessage(&msg,NULL,0,0,PM_REMOVE));	
		ResetEvent(m_hEmptyQueue);
	}
	
	virtual void OnIdle() { Sleep(100); }

};






template<class T>
class CThread : public CThreadBase
{

public:
		
	CThread(BOOL run)
	{

		if (run)
			BeginThread();
	}

	
	BOOL BeginThread(int nTimeout = 1000) 
	{ 
		MYASSERT(!m_running);
		m_id = NULL;
		_beginthreadex(NULL, 0, StaticThreadProc,this, 0,&m_id); 
		m_running = TRUE;

		return (WaitForSingleObject(m_hEventStarted, nTimeout)==WAIT_OBJECT_0);
	}


	void ThreadProc()
	{
		static_cast<T *>(this)->__MainLoop();
		m_running = FALSE;
		SetEvent(m_hEventFinished);		
	}


	static unsigned int __stdcall  StaticThreadProc(void *This)
	{
		static_cast<T *>(This)->ThreadProc();
		return 0;
	}

};




#endif // !defined(AFX_THREAD_H__F73FAE82_F3F2_4497_9786_833B36A6FC4C__INCLUDED_)
