#pragma once
#include <map>
#include "singleton.h"


#define WID_VIEW		0x1
#define WID_MAINFRAME	0x2
#define WID_LVIEW		0x3
#define WID_TVIEW		0x4




//abstract base for creation functor
class CWinCreatorBase
{
public:
	virtual CFrameWindowStuff *Create(CWindow *pParent) const  = 0;
};

namespace winfact_private
{


	//__does__ take ownership of the CWinCreatorBase objects...
	class CWinFactoryPrivate
	{
	private:

		typedef std::map<int, CWinCreatorBase *> maptype;
		maptype m_map;

	public:

		
		
		bool Register(int id, CWinCreatorBase *pCreator)
		{
			m_map.insert(make_pair(id, pCreator));
			return true;
		}

		CFrameWindowStuff *Create(int id, CWindow *pParent = NULL)
		{
			if (m_map.find(id)==m_map.end())
			{
				MessageBox(GetActiveWindow(),_T("Window not registered"),NULL,NULL);
				return 0;
			}

			return m_map[id]->Create(pParent);

		}

		virtual ~CWinFactoryPrivate()
		{
			for (maptype::iterator it = m_map.begin();it!=m_map.end();it++)
				delete it->second;
		}


	};

	
	typedef singleton<winfact_private::CWinFactoryPrivate> CWinFactory;

	template<class X, int i>
	class standard_creator : public CWinCreatorBase
	{
	public:
		static bool m_registered;
		virtual CFrameWindowStuff *Create(CWindow *pParent) const { return new X(pParent);	}
	};


	template<class X, int i>
	bool standard_creator<X, i>::m_registered = CWinFactory::Instance().Register(i, new standard_creator<X,i>);

	
}

typedef singleton<winfact_private::CWinFactoryPrivate> CWinFactory;



//inherit from this
template<class X, int i>
class CCreator
{
	typedef winfact_private::standard_creator<X, i> creator;
	creator m_creator;

	
protected:
	CCreator() { m_creator.m_registered; }
	
};

