// EncSpotList.cpp: implementation of the CEncSpotList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <sys/types.h>
#include <sys/stat.h>

#include <algorithm>

#include <atlcom.h>
#include <atlwin.h>
#include <atldlgs.h>
#include <atlctrls.h>
#include <atlctrlw.h>
#include <atlctrlx.h>
#include <atlctl.h>
#include <atlddx.h>
#include <atlmisc.h>



#include "EncSpotLib\listviewbarctrl.h"

#include "resource.h"


#include "EncSpotList.h"


#ifdef _DEBUG
	int LVITEMDATA::ref_debug = 0;
#endif





#include "EncSpotShellSupport\DropTarget.h"
#include "EncSpotShellSupport\dragsrc.h"
#include "EncSpotShellSupport\dataobj.h"




CEncSpotList::~CEncSpotList()
{
	m_lpiFolder.Free();
	
	try
	{
		m_thread.Close(); 
	}
	catch(...)
	{
		::MessageBox(GetActiveWindow(),_T("List thread is blocked...\nThis is not normal behaviour! :("),NULL,0);
	}
}



HWND CEncSpotList::CreateImp(HWND hWndParent, RECT *pPos)
{
	Create(hWndParent, *pPos, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | LVS_OWNERDATA | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS | LVS_EDITLABELS , 0);	
	SetExtendedListViewStyle(LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP, LVS_EX_LABELTIP | LVS_EX_FULLROWSELECT);
	SetCallbackMask(LVIS_FOCUSED | LVIS_SELECTED);

	//InitListView stuff
	InitColumns();
	
	// Get system image lists
	SHFILEINFO sfi;
	memset(&sfi, 0, sizeof(SHFILEINFO));
	
	HIMAGELIST hImageList = (HIMAGELIST)::SHGetFileInfo(	_T("c:\\"), 0, &sfi, 
															sizeof(sfi),
															SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
	ATLASSERT(hImageList != NULL);
	
	if(hImageList != NULL)
		SetImageList(hImageList, LVSIL_SMALL);

	memset(&sfi, 0, sizeof(SHFILEINFO));
	
	hImageList = (HIMAGELIST)::SHGetFileInfo(_T("c:\\"), 0, &sfi, sizeof(sfi),SHGFI_SYSICONINDEX | SHGFI_ICON);
	ATLASSERT(hImageList != NULL);

	if(hImageList != NULL)
		SetImageList(hImageList, LVSIL_NORMAL);

	UpdateFramesStatus();

	return m_hWnd;
}

LRESULT CEncSpotList::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
	

	m_thread.m_hWndMain = m_hWnd;

	m_pDropTarget = NULL;
	CComObject<CDropTarget>::CreateInstance(&m_pDropTarget);
	m_pDropTarget->AddRef();
	m_pDropTarget->m_hWndMain = m_pMain->m_hWnd;

	RegisterDragDrop(m_hWnd,m_pDropTarget);

	bHandled = FALSE;

	SetTimer(1, 5000);

	return 0;
}

LRESULT CEncSpotList::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
	m_thread.StopEverythingNow();
	StoreColumnWidths();
	m_Header.UnsubclassWindow();
	RevokeDragDrop(m_hWnd);
	m_pDropTarget->Release();

	bHandled = FALSE;
	return 0;
}

tstring CEncSpotList::GetSelectedListContents()
{
	tstring ret;
	vector<int> vItems = GetSelectedItems();
	for (int i = 0;i<vItems.size();i++)
	{
		LPLVITEMDATA lplvid = &GetItemData(vItems[i]);
		ret+=lplvid->path;
		ret+=_T('\0');
	}
	ret+=_T('\0');
	return ret;
}


void CEncSpotList::InitColumns()
{
	
	//ok : count uses m_chids.size()
	for (int i = 0;i<GetColumnCount();i++)
	{
		DWORD id = GetIdOfSubitem(i);

		LVCOLUMN lvc;
		lvc.mask	 =  LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
		lvc.cx		 = m_allcolumns[id].width;
		lvc.pszText  = const_cast<TCHAR *>(m_allcolumns[id].header.c_str());
		lvc.iSubItem = i;

		InsertColumn(lvc.iSubItem, &lvc);
	}
	if (m_Header.m_hWnd)
		m_Header.UnsubclassWindow();

	m_Header.SubclassWindow(GetHeader());
	m_Header.ModifyStyle(0,HDS_BUTTONS | HDS_DRAGDROP | HDS_FULLDRAG |HDS_HOTTRACK,0);
}



LRESULT CEncSpotList::OnColumnClick(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	LPNMLISTVIEW lpLV = (LPNMLISTVIEW)pnmh;
	if(m_idSort == GetIdOfSubitem(lpLV->iSubItem))
		m_bReverseSort = !m_bReverseSort;
	else
		m_idSort = GetIdOfSubitem(lpLV->iSubItem);

	Sort();
	InvalidateRect(NULL);
	
	return 0;
}
	

LRESULT CListSubHeader::OnRClick(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
	CMenuT<true> menu;
	menu.CreatePopupMenu();

	MENUITEMINFO info;
	memset(&info,0,sizeof(info));
	info.cbSize = sizeof(info);
	info.fMask = MIIM_TYPE | MIIM_ID;
	info.fType = MFT_STRING;
	TCHAR str[] = _T("&Customize");
	info.cch   = sizeof(str);
	info.dwTypeData = str;
	info.wID   = 0x7FFF;

	menu.InsertMenuItem(0x7FFF,FALSE,&info);

	POINT pt;
	GetCursorPos(&pt);
	UINT nCmd = TrackPopupMenu (menu, TPM_LEFTALIGN | TPM_LEFTBUTTON |
                             TPM_RIGHTBUTTON | TPM_RETURNCMD, pt.x, pt.y, 0, m_hWnd, NULL);
 
	if (nCmd == 0x7FFF) 
		m_pView->PostMessage(WM_SETTINGS,2);

	return 0;
}

LRESULT CEncSpotList::OnGetDispInfo(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	

	LPNMLVDISPINFO plvdi = reinterpret_cast<LPNMLVDISPINFO>(pnmh);

	MYASSERT(plvdi->item.iItem < size());

	if ( ( -1 == plvdi->item.iItem) || (plvdi->item.iItem >= size()) )
		return 0L;

	LPLVITEMDATA lplvid = &GetItemData(plvdi->item.iItem);
	plvdi->item.iImage = lplvid->iIcon;
	if (plvdi->item.mask & LVIF_TEXT)
	{
	

		int id = GetIdOfSubitem(plvdi->item.iSubItem);

		tstring szText  = GetItemTextById(id, (LPARAM)lplvid);
		plvdi->item.pszText   = const_cast<TCHAR *>(szText.c_str());
	}
	if (plvdi->item.mask & LVIF_STATE)
	{
		plvdi->item.stateMask = LVIS_SELECTED | LVIS_FOCUSED;
		plvdi->item.state	  = 0;
		if (lplvid->bSelected)
			plvdi->item.state |= LVIS_SELECTED;
		if (lplvid->bFocused)
			plvdi->item.state |= LVIS_FOCUSED;

	}		
	return 0L;
}

DWORD CEncSpotList::OnSubItemPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW lpNMCustomDraw)
{
	NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>( lpNMCustomDraw );    
	COLORREF crTextBk;

	LPLVITEMDATA lplvid = &GetItemData(pLVCD->nmcd.dwItemSpec);
	int sub = GetIdOfSubitem(pLVCD->iSubItem);
	int quality = lplvid->data_string.quality;
	if ( (sub == ID_QUALITY) && (quality!=-1) )
	{
		crTextBk = m_quality_cols[lplvid->data_string.quality];
		pLVCD->clrText   = RGB(255,255,255);
	}
	else
	{
		if ( (pLVCD->nmcd.dwItemSpec % 2) == 0 )
			crTextBk = RGB(235,235,255);
		else 
			crTextBk = RGB(255,255,255); 

		pLVCD->clrText   = RGB(0,0,0);
	}

	pLVCD->clrTextBk = crTextBk;
	
	return 0;
}

int CALLBACK CEncSpotList::EncSpotCompareProc(LPARAM lparam1, LPARAM lparam2,LPARAM lparamSort)
{
	LPLVITEMDATA lplvid1=(LPLVITEMDATA)lparam1;
	LPLVITEMDATA lplvid2=(LPLVITEMDATA)lparam2; 

	if (!lplvid1->read && !lplvid2->read)
		return 0;
	else if (!lplvid1->read)
		return (1);
	else if (!lplvid2->read)
		return (-1);

	return CListViewExtremeImpl<CEncSpotList, CListViewVirt<LVITEMDATA> >::ExtremeListViewCompareProc(lparam1,lparam2,lparamSort);
}


void CEncSpotList::Sort() 
{ 
	SortItems(EncSpotCompareProc, (DWORD)this); 
}


LRESULT CEncSpotList::OnRClk(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{


	vector<int> sel = GetSelectedItems();
	if (sel.empty())
		return 0;

	LPCITEMIDLIST *pItemIdArray = new LPCITEMIDLIST[sel.size()];
	
	for (int n = 0;n<sel.size();n++)
	{
		CPidl lpi	= GetItemData(sel[n]).lpi;	
		pItemIdArray[n] = lpi.m_pidl;
	}	

	IShellFolder *psfFolder = GetItemData(sel[0]).spParentFolder;

	CComPtr<IContextMenu> pContextMenu;

	psfFolder->GetUIObjectOf(m_hWnd, sel.size(), pItemIdArray,
                               IID_IContextMenu, NULL, reinterpret_cast<void **>(&pContextMenu));

	CMenu menu_parent;
	menu_parent.LoadMenu(IDR_LVIEWPOPUP);
	CMenuHandle menu = menu_parent.GetSubMenu(0);


	pContextMenu->QueryContextMenu (menu, 3, 1,0x7FFE, CMF_EXPLORE);

	menu.SetMenuDefaultItem(0x7FFF);


	MENUITEMINFO info;
	info.cbSize = sizeof(MENUITEMINFO);
	info.fType = MFT_SEPARATOR;
	info.fMask = MIIM_TYPE;
	menu.InsertMenuItem(3,TRUE,&info);
	

	POINT pt;
	GetCursorPos(&pt);
	UINT nCmd = TrackPopupMenu (menu, TPM_LEFTALIGN | TPM_LEFTBUTTON |
                             TPM_RIGHTBUTTON | TPM_RETURNCMD, pt.x, pt.y, 0, m_hWnd, NULL);

	delete[] pItemIdArray;
 
	
	switch(nCmd)
	{
	case 0:
		break;
	case 0x7FFF:
		{
			for (int i = 0;i<sel.size();i++)
				DoDetails(sel[i]);
		}
		break;
	case 0x8000:
		{
			
			vector<LPLVITEMDATA> *pList = new vector<LPLVITEMDATA>;
			for (int n = 0;n<sel.size();n++)
				pList->push_back(&GetItemData(sel[n]));

			m_pMain->PostMessage(WM_ANIM,1);
			m_thread.PostMessage(WM_PROCESSMULTI, 0, reinterpret_cast<LPARAM>(pList));
		}
		break;
	case 0x8001:
		OnRename(0,0,0,bHandled);
		break;
	default:
		{
			int len = menu.GetMenuStringLen(nCmd,MF_BYCOMMAND);
			TCHAR *pTemp = new TCHAR[++len];
			menu.GetMenuString(nCmd,pTemp,len,MF_BYCOMMAND);
			tstring str = pTemp;
			delete[] pTemp;


			CMINVOKECOMMANDINFO ici;
			ici.cbSize = sizeof (CMINVOKECOMMANDINFO);
			ici.fMask = 0;
			ici.hwnd = m_pMain->m_hWnd;
		
			ici.lpVerb = (char *)MAKEINTRESOURCE (nCmd - 1);
			ici.lpParameters = NULL;
			ici.lpDirectory = NULL;
			ici.nShow = SW_SHOWNORMAL;
			ici.dwHotKey = 0;
			ici.hIcon = NULL;

			pContextMenu->InvokeCommand (&ici);

			SetTimer(0,500);
		}	
		break;
	}
	return 0;
}


//WTL bugfix: case CDDS_ITEMPREPAINT | CDDS_SUBITEM is missing from CCustomDraw<>
LRESULT CEncSpotList::OnCustomDraw(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	CEncSpotList *pT = this;
	pT->SetMsgHandled(TRUE);
	LPNMCUSTOMDRAW lpNMCustomDraw = (LPNMCUSTOMDRAW)pnmh;
	DWORD dwRet = 0;
	switch(lpNMCustomDraw->dwDrawStage)
	{
	case CDDS_PREPAINT:
		dwRet = pT->OnPrePaint(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_POSTPAINT:
		dwRet = pT->OnPostPaint(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_PREERASE:
		dwRet = pT->OnPreErase(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_POSTERASE:
		dwRet = pT->OnPostErase(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_ITEMPREPAINT:
		dwRet = pT->OnItemPrePaint(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_ITEMPOSTPAINT:
		dwRet = pT->OnItemPostPaint(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_ITEMPREERASE:
		dwRet = pT->OnItemPreErase(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_ITEMPOSTERASE:
		dwRet = pT->OnItemPostErase(idCtrl, lpNMCustomDraw);
		break;
	case CDDS_ITEMPREPAINT | CDDS_SUBITEM:
			dwRet = pT->OnSubItemPrePaint(idCtrl, lpNMCustomDraw);
			break;
	default:
		pT->SetMsgHandled(FALSE);
		break;
	}
	bHandled = pT->IsMsgHandled();
	return dwRet;
}

void CEncSpotList::SaveSettings(CIniFile &ini)
{

	ini.SetInt(_T("list_settings"),_T("frameres"),		m_frameresolution);
	ini.SetInt(_T("list_settings"),_T("readallunknown"),m_readallunknown);
	ini.SetInt(_T("list_settings"),_T("readallinitial"),m_readallinitial);
	ini.SetInt(_T("list_settings"),_T("sortcolid"),		m_idSort);
	ini.SetInt(_T("list_settings"),_T("sort_reverse"),	m_bReverseSort);
	ini.SetInt(_T("list_settings"),_T("monoquality"),	m_monoquality);

	ini.SetInt(_T("list_settings"),_T("color_good"),	m_quality_cols[2]);
	ini.SetInt(_T("list_settings"),_T("color_notbad"),	m_quality_cols[1]);
	ini.SetInt(_T("list_settings"),_T("color_bad"),		m_quality_cols[0] );

	//columns

	ini.SetInt(_T("list_settings"),_T("num_columns2"),	GetColumnCount());
	ini.SetInt(_T("list_settings"),_T("autosize"),		m_autosize);
	TCHAR b[10];
	for (int i = 0;i<GetColumnCount();i++)
	{
		DWORD id = GetIdOfSubitem(i);
		tstring tag = _itot(i, b, 10);
		ini.SetInt(_T("list_settings"),_T("colid")+tag,id);
		ini.SetInt(_T("list_settings"),_T("colwidth")+tag,GetColumnWidthById(id));
	}

}
void CEncSpotList::LoadSettings(const CIniFile &ini)
{

	m_frameresolution	= ini.GetInt(_T("list_settings"),_T("frameres"),1000);
	m_readallunknown    = ini.GetInt(_T("list_settings"), _T("readallunknown"), TRUE);
	m_readallinitial	= ini.GetInt(_T("list_settings"), _T("readallinitial"), TRUE);
	m_idSort			= ini.GetInt(_T("list_settings"),_T("sortcolid"),ID_NAME);
	m_bReverseSort		= ini.GetInt(_T("list_settings"),_T("sort_reverse"),0);
	m_monoquality		= ini.GetInt(_T("list_settings"),_T("monoquality"),0);

	m_quality_cols[0] = ini.GetInt(_T("list_settings"),_T("color_bad"),RGB(255,150,150));
	m_quality_cols[1] = ini.GetInt(_T("list_settings"),_T("color_notbad"),RGB(255,255,150));
	m_quality_cols[2] = ini.GetInt(_T("list_settings"),_T("color_good"),RGB(150,255,150));

	//columns

	int numcolumns = ini.GetInt(_T("list_settings"),_T("num_columns2"),0);
	m_autosize  = ini.GetInt(_T("list_settings"),_T("autosize"), TRUE); 
	if (numcolumns == 0)
	{
		DisplayColumn(ID_NAME);
		DisplayColumn(ID_ENCODER);
		DisplayColumn(ID_BITRATE);
		DisplayColumn(ID_FRAMES);
		DisplayColumn(ID_LENGTH);
		DisplayColumn(ID_SIZE);
		DisplayColumn(ID_CUT);
		DisplayColumn(ID_QUALITY);
	}
	else
	{
		TCHAR b[10];
		
		for (int i = 0;i<numcolumns;i++)
		{
			tstring tag = _itot(i, b, 10);
			DWORD id = ini.GetInt(_T("list_settings"), _T("colid")+tag,i);
			int    colwidth = ini.GetInt(_T("list_settings"), _T("colwidth")+tag,100);
			DisplayColumn(id, colwidth);
		}
	}

}


LRESULT CEncSpotList::OnDeleteSelected(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (GetSelectionMark()==-1)
		return 0;

	tstring path = GetSelectedListContents();

	SHFILEOPSTRUCT shop;
	memset(&shop,0,sizeof(shop));
	shop.hwnd = m_hWnd;
	shop.wFunc= FO_DELETE;
	shop.pFrom= path.data();

	BOOL no_success = SHFileOperation(&shop);
	if (!no_success)
		SetTimer(0,500);


	return 0;
}



LRESULT CEncSpotList::OnLVItemKeyDown(int , LPNMHDR pnmh, BOOL& bHandled)
{
	int i = GetSelectionMark();
	if (i==-1)
		return 0;


	LPLVITEMDATA lplvid = &GetItemData(i);

	LPNMLVKEYDOWN pkbd = (LPNMLVKEYDOWN)pnmh;
	switch(pkbd->wVKey)
	{
	case VK_RETURN:
		DoDetails(i);
		break;
	case VK_DELETE:
		PostMessage(WM_DELETE);
		break;
	}
	return 0L;
}



LRESULT CEncSpotList::OnLVDblClk(int , LPNMHDR pnmh, BOOL& )
{
	LPNMITEMACTIVATE lpnmitem = (LPNMITEMACTIVATE) pnmh;
	int i = lpnmitem->iItem;
	if (i==-1)
		return 0;

	DoDetails(i);
	return 0;
}



LRESULT CEncSpotList::OnFill(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	LOG("CEncSpotList::OnFill");

	BOOL delete_all = lParam;

	LPTVITEMDATA pTVI = reinterpret_cast<LPTVITEMDATA>(wParam);
	m_lptvRecentData = pTVI;
	
	m_thread.StopEverythingNow();
	m_pMain->PostMessage(WM_ANIM,0);
	
	LPTVITEMDATA lptvid=pTVI;

	CWaitCursor wait;
	if (lptvid)
	{
		HRESULT hr;
		IShellFolder *spFolder = NULL;
		if (lptvid->root)
			spFolder = lptvid->spParentFolder;
		else
		{
			hr=lptvid->spParentFolder->BindToObject(lptvid->lpi,0,IID_IShellFolder,(LPVOID *)&spFolder);
			if(FAILED(hr)) 
			{
				LOG(_T("CEncSpotList::OnFill Failed to obtain interface to selected folder..."));
				return 0;
			}
		}

	
		LOG(_T("CEncSpotList::OnFill found selected folder interface"));
			
		FillListView(spFolder, delete_all);

		if (!lptvid->root)
			spFolder->Release();
	}
	return 0;
}

LRESULT CEncSpotList::OnBeginLabelEdit(int , LPNMHDR pnmh, BOOL& )
{


	return 0;
}


LRESULT CEncSpotList::OnEndLabelEdit(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	NMLVDISPINFO *pdi = reinterpret_cast<NMLVDISPINFO *>(pnmh);
	TCHAR *szText = pdi->item.pszText;

	if (szText!=NULL)
	{
		int i = GetSelectionMark();
		if (i==-1)
			return 0;

		LPLVITEMDATA lplvid = &GetItemData(i);
		tstring path = lplvid->path;

		
		int pos = path.rfind(_T("\\"));
		tstring newname(path,0,pos+1);

		newname+=szText;

		tstring new_plus = newname + '\0';
		tstring old_plus = path    + '\0';

		SHFILEOPSTRUCT shop;
		memset(&shop, 0, sizeof(shop));
		shop.hwnd = m_hWnd;
		shop.wFunc = FO_RENAME;
		shop.pFrom = old_plus.c_str();
		shop.pTo   = new_plus.c_str();		
		BOOL no_success = SHFileOperation(&shop);
		if (!no_success)	
		{
			SetItemTextById(szText,ID_NAME, (LPARAM)lplvid);
			lplvid->path = newname;
			return TRUE;
		}
	}


	return 0;
}


LRESULT CEncSpotList::OnBeginDrag(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	CComObject<CDragSource> *pDropSource;
	CComObject<CDataObject> *pDataObject;

	int nItems = GetSelectedCount();
	if(!nItems)
	   return 0;

	CComObject<CDragSource>::CreateInstance(&pDropSource);
	if(!pDropSource) return 0;
	pDropSource->AddRef();


	CComObject<CDataObject>::CreateInstance(&pDataObject);
	if(!pDataObject)
	{
	   pDropSource->Release();
	   return 0;
	}
	pDataObject->AddRef();
		  
	

	tstring text = GetSelectedListContents();
	pDataObject->Setup(NULL,text);


	// do the drag and drop
	DWORD dwEffect;
	HRESULT  hr = DoDragDrop(pDataObject, pDropSource, DROPEFFECT_COPY | DROPEFFECT_MOVE, &dwEffect);
	if (hr==DRAGDROP_S_DROP)
		SetTimer(0,500);	
	
	pDataObject->Release();
	pDropSource->Release();

	
	return 0;
}

void CEncSpotList::InitPropPages(BOOL *pDirty)
{

	//general
	m_prop_list_general.m_frameres		= m_frameresolution;
	m_prop_list_general.m_readallunknown= m_readallunknown;
	m_prop_list_general.m_initial		= m_readallinitial;

	m_prop_list_general.m_pDirty = pDirty;
	m_prop_list_quality.m_pDirty = pDirty;
	m_prop_list_columns.m_pDirty = pDirty;



	//quality tab
	memcpy(m_prop_list_quality.m_cols, m_quality_cols, sizeof(m_quality_cols));
	m_prop_list_quality.m_monoquality = m_monoquality;


	//columns tab
	m_prop_list_columns.m_chcolumns.clear();
	m_prop_list_columns.m_avcolumns.clear();

	m_prop_list_columns.m_autosize  = m_autosize;
	for (int i = 0;i<GetColumnCount();i++)
	{
		int id = GetIdOfSubitem(i);
		tstring coltext = GetColumnHeaderById(id);
		m_prop_list_columns.m_chcolumns.push_back(coltext);
	}
	for (i = 0; i<GetColumnPoolSize(); i++)
	{
		int id = GetId(i);
		tstring coltext = GetColumnHeaderById(id);
		if (!IsColumnVisible(id))
			m_prop_list_columns.m_avcolumns.push_back(coltext);
	}


}

void CEncSpotList::ApplyPropPageChanges(BOOL dirty)
{
	//general
	m_frameresolution = m_prop_list_general.m_frameres;
	m_readallunknown  = m_prop_list_general.m_readallunknown;
	m_readallinitial  = m_prop_list_general.m_initial;

	//quality
	m_monoquality = m_prop_list_quality.m_monoquality;
	memcpy(m_quality_cols, m_prop_list_quality.m_cols, sizeof(m_quality_cols));
	m_autosize    = m_prop_list_columns.m_autosize;

	//columns
	DeleteAllColumns();
	for (int i = 0;i<m_prop_list_columns.m_chcolumns.size();i++)
	{
		DWORD id = GuessIdFromHeader(m_prop_list_columns.m_chcolumns[i]);
		DisplayColumn(id, GetColumnWidthById(id));
	}
	InitColumns();

	//refresh
	if (dirty)
		SendMessage(WM_REFRESH);
	else
	{
		if (m_autosize)
			AutoSize(0,200);
	}

	UpdateFramesStatus();

}

void CEncSpotList::AddPropPages(CSettingsDlg *psheet)
{
	psheet->AddPage(m_prop_list_general);
	psheet->AddPage(m_prop_list_quality);
	psheet->AddPage(m_prop_list_columns);

}



bool CEncSpotList::IsMp3(const tstring &path)
{
	LPCTSTR pExt = PathFindExtension(path.c_str());
	if (*pExt == '\0')
		return false;
		
	CString ext = pExt;
	ext.MakeLower();
	return (ext == _T(".mp3"));
}



//note does _not_ free lpi
LPLVITEMDATA CEncSpotList::AddItemToList(const CPidl &lpi, IShellFolder *pShellFolder, CComPtr<IMalloc> spMalloc)
{
	tstring path = lpi.GetPath(pShellFolder);
	
	LOG(_T("CEncSpotList::AddItemToList:\t")+path);
	
	if (!IsMp3(path))
		return NULL;

	//icon stuff: calculate once only...
	static int iIcon = -1;
	if (iIcon == -1)
	{
		CPidl abs_pidl = lpi.GetFullyQualPidl(pShellFolder);
		iIcon = abs_pidl.GetIcon(SHGFI_PIDL | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
		abs_pidl.Free();
	}	

	//adds data to _virtual_ list. ItemCount is incremented
	//afterwards, in FillListView to notify the control of this change
	LPLVITEMDATA lplvid = &CreateNewData();
	
	lplvid->iIcon = iIcon;
	lplvid->spParentFolder=pShellFolder;
	lplvid->spParentFolder->AddRef();

	// Now make a copy of the ITEMIDLIST.
	lplvid->lpi.Copy(lpi,spMalloc);

	//name
	tstring name = lpi.GetName(pShellFolder);
	lplvid->name = name;
	lplvid->path = path;

	return lplvid;	
}


BOOL CEncSpotList::FillListView(IShellFolder* pShellFolder, BOOL delete_all)
{
	LOG(_T("CEncSpotList::FillListView"));
	if(!pShellFolder) return FALSE;

	m_bDoneThorough = FALSE;
	m_bDropMode     = FALSE;

	++m_dwCookie;

	if( (size() > 0) && (delete_all))
		DeleteAllItems();

	//get Enum interface
	CComPtr<IEnumIDList>	spEnumIDList;	
	HRESULT hr=pShellFolder->EnumObjects(GetParent(),SHCONTF_NONFOLDERS, &spEnumIDList);
	if (FAILED(hr))
	{
		LOG(_T("Failed to get IEnumIDList interface..."));
		return FALSE;
	}
	LOG(_T("CEncSpotList::FillListView obtained IEnumIDList interface"));

	//add items 
	ULONG        ulFetched =0;
	CPidl lpi;		

	CComPtr<IMalloc> spMalloc;
	SHGetMalloc(&spMalloc);
	vector<LPLVITEMDATA> *pLvidlist = new vector<LPLVITEMDATA> ;
	while (S_OK==spEnumIDList->Next(1, &lpi.m_pidl, &ulFetched))
	{
		LPLVITEMDATA lvid = AddItemToList(lpi, pShellFolder, spMalloc);	
		lpi.Free();

		if (lvid!=NULL)
			pLvidlist->push_back(lvid);

	}

	if (pLvidlist->size())
		m_pMain->PostMessage(WM_ANIM, 1);

	WPARAM wParam = MAKELONG(m_frameresolution, m_dwCookie);

	m_thread.PostMessage(WM_PROCESSMULTI, wParam,(LPARAM)pLvidlist);

	return TRUE;
}


LRESULT CEncSpotList::OnThreadDone(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	DWORD dwCookie = HIWORD(wParam);
	BOOL none_left = LOWORD(wParam);
	
	
	
	LPLVITEMDATA lplvid = reinterpret_cast<LPLVITEMDATA>(lParam);
	LOG(_T("CEncSpotList::OnThreadDone: inserting item  ")+lplvid->name);

	lplvid->read = true;
	
	int nSoFar = 0;
	for (int i = 0;i<size();i++)
	{
		LPLVITEMDATA lpv = &GetItemData(i);
		if (lpv->read)
			nSoFar++;
	}

	SetItemCount(nSoFar);

	int nThis	= FindItemByData(*lplvid);
	int nInsert = FindListViewInsertionPos(lplvid);
	
	ReinsertOurItem(nThis, nInsert);

	if (none_left)
		m_pMain->PostMessage(WM_ANIM,0);


	if (nSoFar==size()) 
	{
		UpdateDirStatus();
		if (m_autosize)
			AutoSize(0,200);

		if (m_readallinitial)
			DoThoroughScan();

	//	RedrawItems(0, GetItemCount()-1);


		
	}
	else
		UpdateProgressStatus(nSoFar);

	return 0;
}



LRESULT CEncSpotList::OnRefresh(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	if (m_lptvRecentData)
		SendMessage(WM_FILL, reinterpret_cast<WPARAM>(m_lptvRecentData), TRUE);
	return 0;
}

LRESULT CEncSpotList::OnTimer(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	if (wParam==0)
		KillTimer(0);

	if (m_bDropMode)
		return 0;


	
	vector<LPLVITEMDATA> items_to_add;
	CComPtr<IMalloc>        spMalloc;
	if (FAILED(SHGetMalloc(&spMalloc)))
		return 0;


	vector<tstring> disk_contents0 = GetFolderContents();
	vector<tstring> disk_contents  = disk_contents0;
	vector<CPidl>	disk_pidls    = GetFolderPidls();

	vector<tstring> list_contents;
	for (int i = 0;i<size();i++)
	{
		LPLVITEMDATA lplvi = &GetItemData(i);
		tstring name = lplvi->name;
		list_contents.push_back(name);
	}

	
	IShellFolder *spFolder = NULL;
	if (m_lptvRecentData->root)
		spFolder = m_lptvRecentData->spParentFolder;
	else
	{
		HRESULT hr=m_lptvRecentData->spParentFolder->BindToObject(m_lptvRecentData->lpi,0,IID_IShellFolder,(LPVOID *)&spFolder);
		if(FAILED(hr)) 
		{
			LOG(_T("CEncSpotList::GetFolderContents Failed to obtain interface to selected folder..."));
			return 0;
		}
	}

		
	for(;;)
	{
		sort(disk_contents.begin(), disk_contents.end());
		sort(list_contents.begin(), list_contents.end());

		for (int i = 0;i<list_contents.size();i++)
		{
			const tstring &list = list_contents[i];
			if ( (i >= disk_contents.size()) || (list!= disk_contents[i]))
			{
				if (find(disk_contents.begin(), disk_contents.end(), list)==disk_contents.end())
				{
					//case 1: item has been _removed_ from directory, but remains on the list
					for (int j = 0;j<size();j++)
					{
						LPLVITEMDATA lplvi = &GetItemData(j);
						if (lplvi->name == list)
							break;
					}
					if (j<size())
					{
						list_contents.erase(list_contents.begin()+i);
						DeleteItem(j);
					}	
					break;
				}
			}
		}

		if (i < list_contents.size())
			continue;

		for (i = 0;i<disk_contents.size();i++)
		{
			const tstring &disk = disk_contents[i];
			if ( (i >= list_contents.size()) || (disk!= list_contents[i]))
			{
				tstring test = list_contents[i];
				//case 2: item has been _added_ to directory, not yet on list
				//case 1: item has been _removed_ from directory, but remains on the list
				for (int j = 0;j<disk_contents0.size();j++)
				{
					if (disk == disk_contents0[j])
						break;
				}

				CPidl lpi = disk_pidls[j];
				LPLVITEMDATA lplvi = AddItemToList(lpi, spFolder, spMalloc);
				MYASSERT(lplvi!=NULL);
				
				items_to_add.push_back(lplvi);
				list_contents.push_back(disk);
				break;
			
			}
		}

		if (i==disk_contents.size())
			break;
	}//for(;;)


	for (i = 0;i<disk_pidls.size();i++)
			disk_pidls[i].Free();

	if (!m_lptvRecentData->root)
		spFolder->Release();


	if (items_to_add.size())
	{
		vector<LPLVITEMDATA> *pList = new vector<LPLVITEMDATA>(items_to_add);

		m_pMain->PostMessage(WM_ANIM, 1);
		m_thread.PostMessage(WM_PROCESSMULTI, MAKELONG(m_frameresolution, m_dwCookie),(LPARAM)pList);
	}



	return 0;
}
LRESULT CEncSpotList::OnStop(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	m_thread.StopEverythingNow();
	m_pMain->PostMessage(WM_ANIM,0);
	return 0;
}
	

void CEncSpotList::UpdateProgressStatus(int nSoFar)
{
	TCHAR szNum[256];
	_stprintf(szNum, _T("%i read, %i to go"), nSoFar, size() - nSoFar);
	PostStatusMessage(0,szNum);
}


void CEncSpotList::UpdateDirStatus()
{

	TCHAR szNum[256];
	int count = GetItemCount();

	float length	= 0;
	float fTotal = 0;
	float fTotalSize = 0;
	for (int i = 0;i<count;i++)
	{
		LVITEMDATA &data = GetItemData(i);
		length+=data.data.length;
		fTotal += data.data.bitrate * data.data.length;
		fTotalSize+=data.data.fsize;
	}

	float average_bitrate = 0;
	if (length > 0)
		average_bitrate = fTotal / length;

	TCHAR units[3];
	lstrcpy(units, _T("MB"));
	float size = fTotalSize/(1024*1024);
	if (size > 1024)
	{
		size/=1024;
		lstrcpy(units, _T("GB"));
	}


	if (count != 1)
		_stprintf(szNum, _T("%i files, %.1f %s, %.1f kbps"), count, size,units,average_bitrate);
	else
		_stprintf(szNum, _T("1 file, %.1f %s, %.1f kbps"), size, units, average_bitrate);

	PostStatusMessage(0,szNum);

}

LRESULT CEncSpotList::OnRename(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	int i = GetSelectionMark();
	if (i == -1)
		return 0;

	EditLabel(i);

	return 0;
}


void CEncSpotList::PostStatusMessage(int panel, tstring msg)
{
	tstring *pStr = new tstring(msg);
	m_pMain->PostMessage(WM_STATUS, panel, reinterpret_cast<LPARAM>(pStr));

}

void CEncSpotList::UpdateFramesStatus()
{
	TCHAR b[256];
	tstring num;
	if (m_frameresolution)
		_stprintf(b,_T("Reading at most %i frames"), m_frameresolution);
	else
		_stprintf(b,_T("Reading all frames"));

	PostStatusMessage(1,b);

}


int CEncSpotList::FindListViewInsertionPos(LPLVITEMDATA lplvid)
{
	for (int i = 0;i<size();i++)
	{
		LPARAM param2 = (LPARAM)(&GetItemData(i));
		if (CEncSpotList::EncSpotCompareProc((LPARAM)lplvid, param2,(LPARAM)this) < 0)
			break;
	}
	return i;
}


void CEncSpotList::DoDetails(int i)
{
	LPLVITEMDATA lplvid = &GetItemData(i);
	lplvid->path = lplvid->lpi.GetPath(lplvid->spParentFolder);
	lplvid->name = lplvid->lpi.GetName(lplvid->spParentFolder);


	LPARAM lParam = reinterpret_cast<LPARAM>(new tstring(lplvid->path));

	if (lplvid->data.all_read)
		m_pMain->PostMessage(WM_DETAILS,reinterpret_cast<WPARAM>(new mp3data(lplvid->data)),lParam);
	else
		m_pMain->PostMessage(WM_DETAILS,NULL, lParam);
}


LRESULT CEncSpotList::OnExport(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{

	TCHAR ext[]  = _T(".xml\0\0");
	TCHAR f[] = _T("XML Files\0*.xml\0Text Files\0*.txt\0\0");
	CFileDialog aDlg(FALSE, ext,NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, f);
	if (aDlg.DoModal()==IDOK)
	{
		tstring fname = aDlg.m_szFileName;
		tstring ext   = PathFindExtension(aDlg.m_szFileName);
		tstring out;
		if (ext==".txt")
		{
			out = ExportTextFile();
		}
		else if (ext==".xml")
			out = ExportXML();
		else
		{
			MessageBox(_T("Unrecognized filetype"));
			return 0;
		}

		FILE *pFile = NULL;
		pFile = fopen(fname.c_str(), _T("wb"));
		if (pFile==NULL)
		{
			MessageBox("Unable to write to file");
		}
		else
		{
			fwrite(out.c_str(), 1, out.size(), pFile);
			fclose(pFile);

			::ShellExecute(m_hWnd, _T("open"), fname.c_str(), 0, 0, SW_SHOWNORMAL);
		}
	}

	
	

	return 0;
}


tstring CEncSpotList::ExportTextFile()
{
	tstring out;
		
	for (int i = 0;i<GetColumnCount();i++)
	{
		DWORD id = GetIdOfSubitem(i);
		tstring header = GetColumnHeaderById(id);
		out+=header + tstring(max(30 - header.size(),0),_T(' '));
	}
	out+=_T("\r\n");

	for (i = 0;i<GetColumnCount();i++)
	{
		DWORD id = GetIdOfSubitem(i);
		tstring header = GetColumnHeaderById(id);
		out+=tstring(header.size(), _T('-')) + tstring(max(30 - header.size(),0),_T(' '));
	}
	out+=_T("\r\n");

	for (int n = 0;n<GetItemCount();n++)
	{
		for (int i = 0;i<GetColumnCount();i++)
		{
		
			DWORD id = GetIdOfSubitem(i);
			LPLVITEMDATA lpv = &GetItemData(n);
			tstring data = GetItemTextById(id, (LPARAM)lpv);

			if (id==ID_QUALITY)
			{
				int qual = lpv->data_string.quality;
				if (qual == 0)
					data = _T("bad");
				else if (qual==1)
					data = _T("fair");
				else if (qual==2)
					data = _T("good");
			}



			if (data.size() > 30)
			{
				tstring temp = data.substr(0,27);
				data = temp + _T("...");
			}
			int space = (data.size()>30)?1:(30-data.size());
			out+=data + tstring(space,_T(' '));
		}
		out+=_T("\r\n");
	}

	return out;

}

tstring CEncSpotList::ExportXML()
{
	tstring out = _T("<?xml version=\"1.0\"?>\r\n");
	out+=_T("<encpsot>\r\n");
	for (int n = 0;n<GetItemCount();n++)
	{
		out+=_T("\t<file>\r\n");
		for (int i = 0;i<GetColumnCount();i++)
		{
			DWORD id = GetIdOfSubitem(i);
			tstring header = GetColumnHeaderById(id);

			//get better form
			int pos = header.find(_T(' '));
			if (pos!=string::npos)
			{
				tstring first = header.substr(0,pos);
				pos = header.rfind(_T(' '));
				tstring second= header.substr(pos+1,string::npos);
				
				if (isalnum(second[0]))
					header = second;
				else
					header = first;
				
			}

			

			LPLVITEMDATA lpv = &GetItemData(n);
			tstring data = GetItemTextById(id, (LPARAM)lpv);

			if (id==ID_QUALITY)
			{
				int qual = lpv->data_string.quality;
				if (qual == 0)
					data = _T("bad");
				else if (qual==1)
					data = _T("fair");
				else if (qual==2)
					data = _T("good");
			}

			out+=_T("\t\t<") + header + _T(">");

			out+=data;

			out+=_T("</") + header + _T(">\r\n");
			




		}
		out+=_T("\t</file>\r\n");
	}
	out+=_T("</encpsot>\r\n");
	return out;
}


LRESULT CEncSpotList::OnDrop(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	LOG(_T("CEncSpotList::OnDrop"));

	m_thread.StopEverythingNow();
	m_pMain->PostMessage(WM_ANIM,0);


	USES_CONVERSION;


	
	DWORD grfKeyState  = wParam;
	vector<tstring> *pFilename = reinterpret_cast<vector<tstring> *>(lParam);

	vector<tstring> filenames = *pFilename;
	delete pFilename;

	if (filenames.empty())
		return 0;

	
	if ((grfKeyState & MK_CONTROL) && !(grfKeyState & MK_SHIFT))
	{
		FileOperation(filenames, FALSE);
		return 0;
		
	}
	else if (!(grfKeyState & MK_CONTROL) && (grfKeyState & MK_SHIFT))
	{
		FileOperation(filenames, TRUE);
		return 0;
	}

	//otherwise just display the files.

	m_bDropMode = TRUE;
	

	CComPtr<IMalloc>        spMalloc;
	if (FAILED(SHGetMalloc(&spMalloc)))
		return 0;

	CComPtr<IShellFolder> spFolder;

	HRESULT hr = SHGetDesktopFolder(&spFolder);
	if (FAILED(hr))
		return 0;

	DeleteAllItems();


	vector<LPLVITEMDATA> *pLvidlist = new vector<LPLVITEMDATA> ;
	
	for (int i = 0;i<filenames.size();i++)
	{
		CPidl fullpidl;
		ULONG	parsed = 0;
		tstring path = filenames[i];
		if (!IsMp3(path))
			continue;

		
		HRESULT hr = spFolder->ParseDisplayName(m_hWnd,NULL,T2W(path.c_str()),&parsed, &fullpidl.m_pidl, NULL);
		if(SUCCEEDED(hr))
		{
			CPidl pParentPidl;
			pParentPidl.CreateParentPidl(fullpidl, spMalloc);
			CPidl pRelativePidl;
			pRelativePidl.CreateRelativePidl(fullpidl, spMalloc);
			int len = pRelativePidl.length();

			IShellFolder *pParentFolder = NULL;
			hr = spFolder->BindToObject(pParentPidl, 0, IID_IShellFolder, (void **)&pParentFolder);
			if (SUCCEEDED(hr))
			{

				LPLVITEMDATA lvid = AddItemToList(pRelativePidl, pParentFolder, spMalloc);
				if (lvid!=NULL)
					pLvidlist->push_back(lvid);
			}

			pRelativePidl.Free();
			pParentPidl.Free();
			pParentFolder->Release();
		}
		fullpidl.Free();
	}
	

	if (pLvidlist->size())
		m_pMain->PostMessage(WM_ANIM, 1);

	m_thread.PostMessage(WM_PROCESSMULTI, MAKELONG(m_frameresolution, m_dwCookie),(LPARAM)pLvidlist);
	return 0;
}

LRESULT CEncSpotList::FileOperation(vector<tstring> fnames, BOOL bMove)
{

	tstring path;
	
	if (m_lptvRecentData->root)
		path = get_desktop_path();
	else
	{
		TCHAR szPath[MAX_PATH];
	

		LPTVITEMDATA lptv = m_lptvRecentData;
		CPidl pidlabs = lptv->lpi.GetFullyQualPidl(m_lptvRecentData->spParentFolder);
		SHGetPathFromIDList(pidlabs.m_pidl, szPath);
		pidlabs.Free();
		path = szPath;
	}

	tstring buffer;

	for (int i = 0;i<fnames.size();i++)
	{
		buffer+=fnames[i];
		buffer+=_T('\0');

	}
	buffer+=_T('\0');
	buffer+=_T('\0');

	SHFILEOPSTRUCT shop;
	memset(&shop, 0, sizeof(shop));
	shop.hwnd = m_hWnd;
	shop.wFunc = bMove?FO_MOVE:FO_COPY;
	shop.pFrom = buffer.data();
	shop.pTo   = path.c_str();
	BOOL no_success = SHFileOperation(&shop);
	if (!no_success)
		return TRUE;


	
	return FALSE;
}

LRESULT CEncSpotList::OnBadFile(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	LPLVITEMDATA lvi = reinterpret_cast<LPLVITEMDATA>(wParam);
	int i = FindItemByData(*lvi);
	if (i==-1)
		return 0;

	LOG(_T("CEncSpotList::OnBadFile"));

	DeleteItem(i);
	

	return 0;
}

void CEncSpotList::DoThoroughScan()
{
	if (m_bDoneThorough)
		return;

	m_bDoneThorough = TRUE;


	vector<LPLVITEMDATA> *pList = new vector<LPLVITEMDATA>;
	for (int n = 0;n<size();n++)
	{
		LPLVITEMDATA lvi = &GetItemData(n);
		if (!lvi->data.all_read)
			pList->push_back(lvi);
	}

	if (pList->size())
	{
		m_pMain->PostMessage(WM_ANIM,1);
		m_thread.PostMessage(WM_PROCESSMULTI, 0, reinterpret_cast<LPARAM>(pList));
	}
	else
		delete pList;

}


//perform clipboard operations
LRESULT CEncSpotList::OnShellOp(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{


	WORD wId = wParam;
	vector<int> sel = GetSelectedItems();

	switch(wId)
	{
	case ID_EDIT_COPY:
		{
			CComObject<CDataObject> *pDataObject = NULL;

			CComObject<CDataObject>::CreateInstance(&pDataObject);
			if(!pDataObject)
			   return 0;
			pDataObject->AddRef();
			tstring text = GetSelectedListContents();

			CLIPFORMAT cf = RegisterClipboardFormat(CFSTR_PREFERREDDROPEFFECT);
			pDataObject->Setup(cf,text, TRUE, TRUE);

			OleSetClipboard(pDataObject);

			pDataObject->Release();

		}
		break;

	case ID_EDIT_CUT:
		{
			CComObject<CDataObject> *pDataObject = NULL;

			CComObject<CDataObject>::CreateInstance(&pDataObject);
			if(!pDataObject)
			   return 0;
			pDataObject->AddRef();
			tstring text = GetSelectedListContents();

			CLIPFORMAT cf = RegisterClipboardFormat(CFSTR_PREFERREDDROPEFFECT);
			pDataObject->Setup(cf,text, TRUE, FALSE);

			OleSetClipboard(pDataObject);

			pDataObject->Release();

		}
		break;


	case ID_EDIT_PASTE:
		{
			CComPtr<IDataObject> pDataObject = NULL;
			OleGetClipboard(&pDataObject);

			FORMATETC form;
			form.cfFormat	= CF_HDROP;
			form.ptd		= NULL;
			form.dwAspect   = DVASPECT_CONTENT;
			form.lindex		= -1;
			form.tymed		= TYMED_HGLOBAL;

			HRESULT hr = pDataObject->QueryGetData(&form);
			if (FAILED(hr))
				break;

			STGMEDIUM stg;

			hr = pDataObject->GetData(&form, &stg);
			if (FAILED(hr))
				break;

			HDROP hDrop = (HDROP)stg.hGlobal;
			vector<tstring> files;
			int count = DragQueryFile(hDrop,-1,NULL,NULL);
			for (int n = 0;n<count;n++)
			{
				TCHAR b[MAX_PATH];
				DragQueryFile(hDrop,n,b,MAX_PATH);
				tstring mp3 = b;
				if (IsMp3(mp3))
					files.push_back(mp3);
			}
				
			ReleaseStgMedium(&stg);

			//work out whether move or copy
			CLIPFORMAT cf = RegisterClipboardFormat(CFSTR_PREFERREDDROPEFFECT);
			
			form.cfFormat = cf;
			hr = pDataObject->GetData(&form, &stg);
			if (FAILED(hr))
				break;

			DWORD *pEffect = (DWORD *)GlobalLock(stg.hGlobal);
			DWORD effect = *pEffect;
			GlobalUnlock(stg.hGlobal);

			ReleaseStgMedium(&stg);

			if (effect & DROPEFFECT_COPY)
				FileOperation(files, FALSE);
			else if (effect & DROPEFFECT_MOVE)
				FileOperation(files, TRUE);
		}
		break;


	}

	return 0;
}


vector<tstring>	CEncSpotList::GetFolderContents()
{
	vector<tstring> ret;


	IShellFolder *spFolder = NULL;
	if (m_lptvRecentData->root)
		spFolder = m_lptvRecentData->spParentFolder;
	else
	{
		HRESULT hr=m_lptvRecentData->spParentFolder->BindToObject(m_lptvRecentData->lpi,0,IID_IShellFolder,(LPVOID *)&spFolder);
		if(FAILED(hr)) 
		{
			LOG(_T("CEncSpotList::GetFolderContents Failed to obtain interface to selected folder..."));
			return 0;
		}
	}
		
	//get Enum interface
	CComPtr<IEnumIDList>	spEnumIDList;	
	HRESULT hr=spFolder->EnumObjects(GetParent(),SHCONTF_NONFOLDERS, &spEnumIDList);
	if (FAILED(hr))
	{
		LOG(_T("CEncSpotList::GetFolderContents failed to get IEnumIDList interface..."));
		return FALSE;
	}

	CPidl lpi;
	ULONG ulFetched =0;
	while (S_OK==spEnumIDList->Next(1, &lpi.m_pidl, &ulFetched))
	{
		tstring name = lpi.GetName(spFolder);
		tstring path = lpi.GetPath(spFolder);
		if (IsMp3(path))
			ret.push_back(name);
		lpi.Free();
	}

	if (!m_lptvRecentData->root)
		spFolder->Release();

	return ret;
}


vector<CPidl>	CEncSpotList::GetFolderPidls()
{
	vector<CPidl> ret;


	IShellFolder *spFolder = NULL;
	if (m_lptvRecentData->root)
		spFolder = m_lptvRecentData->spParentFolder;
	else
	{
		HRESULT hr=m_lptvRecentData->spParentFolder->BindToObject(m_lptvRecentData->lpi,0,IID_IShellFolder,(LPVOID *)&spFolder);
		if(FAILED(hr)) 
		{
			LOG(_T("CEncSpotList::GetFolderPidls Failed to obtain interface to selected folder..."));
			return 0;
		}
	}
	
	//get Enum interface
	CComPtr<IEnumIDList>	spEnumIDList;	
	HRESULT hr=spFolder->EnumObjects(GetParent(),SHCONTF_NONFOLDERS, &spEnumIDList);
	if (FAILED(hr))
	{
		LOG(_T("CEncSpotList::GetFolderPidls failed to get IEnumIDList interface..."));
		return FALSE;
	}

	CPidl lpi;
	ULONG ulFetched =0;
	while (S_OK==spEnumIDList->Next(1, &lpi.m_pidl, &ulFetched))
	{
		tstring name = lpi.GetPath(spFolder);
		if (IsMp3(name))
			ret.push_back(lpi);
		else
			lpi.Free();
	}

	if (!m_lptvRecentData->root)
		spFolder->Release();

	return ret;
}

LRESULT CEncSpotList::OnSelectAll(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetFocus();
	for (int i = 0;i<GetItemCount();i++)
	{
		LVITEM lvi;
		lvi.stateMask = LVIS_SELECTED;
		lvi.state     = LVIS_SELECTED;
		
		BOOL s = SetItemState(i, &lvi);
		DWORD err = GetLastError();
	}
	return 0;
}

LRESULT CEncSpotList::OnItemsChanged(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	LPNMLVODSTATECHANGE lpnmlv = (LPNMLVODSTATECHANGE)pnmh;

	UINT iFrom = lpnmlv->iFrom;
	UINT iTo   = lpnmlv->iTo;

	for (int i = iFrom;i<=iTo;i++)
	{
		LPLVITEMDATA lpd = &GetItemData(i);
		if ((lpnmlv->uNewState & LVIS_SELECTED) != (lpnmlv->uOldState & LVIS_SELECTED))
		{
			if (lpnmlv->uNewState & LVIS_SELECTED)
				lpd->bSelected = TRUE;
			else
				lpd->bSelected = FALSE;
		}

		if ((lpnmlv->uNewState & LVIS_FOCUSED )!= (lpnmlv->uOldState & LVIS_FOCUSED))
		{
			if (lpnmlv->uNewState & LVIS_FOCUSED)
				lpd->bFocused = TRUE;
			else
				lpd->bFocused = FALSE;
		}
	}

	RefreshVisibleStates();
			

	bHandled = FALSE;
	return 0;
}


LRESULT CEncSpotList::OnItemChanged(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	LPNMLISTVIEW lpnmlv = (LPNMLISTVIEW)pnmh;

	if (lpnmlv->iItem==-1)
	{
		//applies to all items.
		
		NMLVODSTATECHANGE nmlv;
		nmlv.hdr	= lpnmlv->hdr;
		nmlv.iFrom	= 0;
		nmlv.iTo	= GetItemCount()-1;
		nmlv.uNewState = lpnmlv->uNewState;
		nmlv.uOldState = lpnmlv->uOldState;
		OnItemsChanged(0, (LPNMHDR)&nmlv, bHandled);
		return 0;

	}

	LPLVITEMDATA lpd = &GetItemData(lpnmlv->iItem);
	if ((lpnmlv->uNewState & LVIS_SELECTED) != (lpnmlv->uOldState & LVIS_SELECTED))
	{
		if (lpnmlv->uNewState & LVIS_SELECTED)
			lpd->bSelected = TRUE;
		else
			lpd->bSelected = FALSE;
	}

	if ((lpnmlv->uNewState & LVIS_FOCUSED )!= (lpnmlv->uOldState & LVIS_FOCUSED))
	{
		if (lpnmlv->uNewState & LVIS_FOCUSED)
		{
			for (int j = 0;j<GetItemCount();j++)
			{
				if (j!=lpnmlv->iItem)
					GetItemData(j).bFocused = FALSE;
			}
			lpd->bFocused = TRUE;
		}
		else
			lpd->bFocused = FALSE;
	}
	
	RefreshVisibleStates();
			
	bHandled = FALSE;
	return 0;
}


	

vector<int> CEncSpotList::GetSelectedItems()
{	
	vector<int> ret;
	for (int i = 0;i<GetItemCount();i++)
	{
		LPLVITEMDATA pdata = &GetItemData(i);
		if (pdata->bSelected)
			ret.push_back(i);
	}
	return ret;
}

void CEncSpotList::RefreshVisibleStates()
{
	for (int i = 0;i<GetItemCount();i++)
	{
		LVITEM lvi;
		lvi.mask = LVIF_STATE;
		lvi.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
		lvi.state = 0;

		if (GetItemData(i).bSelected)
			lvi.state|=LVIS_SELECTED;
		if (GetItemData(i).bFocused)
			lvi.state|=LVIS_FOCUSED;


		SetItemState(i, &lvi);
	}

}
