// EncSpotList.h: interface for the CEncSpotList class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ENCSPOTLIST_H__70C7F02F_06BB_11D5_A176_080046068885__INCLUDED_)
#define AFX_ENCSPOTLIST_H__70C7F02F_06BB_11D5_A176_080046068885__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <map>

#include "itemdata.h"
#include "ListThread.h"

#include "EncSpotLib\listviewextreme.h"
#include "EncSpotLib\interface.h"
#include "EncSpotLib\winfactory.h"

class CDropTarget;  

//to us from CReadAllDlg
//#define WM_NOTIFY_READDLG WM_USER+12

//to CEncSpotView
#define WM_DELETE		WM_USER+15
#define WM_BADFILE		WM_USER+40
#define WM_DROP			WM_USER+4
#define WM_SETTINGS		WM_USER+11
#define WM_ANIM			WM_USER+31
#define WM_DETAILS		WM_USER+32
						
//from CEncSpotView		
#define WM_FILL			WM_USER+17
#define WM_REFRESH		WM_USER+13
#define WM_STOP			WM_USER+20
#define WM_RENAME		WM_USER+14
#define WM_STATUS		WM_USER+30
#define WM_EXPORT		WM_USER+21
#define WM_SHELLOP		WM_USER+41
#define WM_SELECTALL	WM_USER+42


//from thread
#define WM_THREADDONE	WM_USER+6


#define ID_NAME			0
#define ID_ENCODER		1 
#define ID_BITRATE		2
#define ID_MODE			3
#define ID_LENGTH		4
#define ID_SIZE			5
#define ID_CUT			6 
#define ID_QUALITY		7
#define ID_FREQUENCY	8
#define ID_FRAMES		9
#define ID_RESERVOIR	10
#define ID_TYPE			11
#define ID_VBR			12
#define ID_CUE			13

#include "PropList.h"
#include "stdafx.h"	// Added by ClassView

class CListSubHeader : public CWindowImpl<CHeaderCtrl>
{
	CWindow *m_pView;
public:
	CListSubHeader(CWindow *pV) : m_pView(pV) {}

	BEGIN_MSG_MAP(CListSubHeader)
		MESSAGE_HANDLER(WM_RBUTTONUP,OnRClick)
	END_MSG_MAP()

	LRESULT OnRClick(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
}; 

class CEncSpotList :	public CListViewExtremeImpl<CEncSpotList, CListViewVirt<LVITEMDATA> >,
						public CCustomDraw<CEncSpotList>,
						public CWindowWithSettings,
						public CFrameWindowStuffImpl<CEncSpotList>, 
						public CCreator<CEncSpotList, WID_LVIEW>
{
	CComObject<CDropTarget> *	m_pDropTarget;
	CWindow *					m_pMain;
	CListSubHeader				m_Header;	
	LPTVITEMDATA				m_lptvRecentData;	
	CListThread					m_thread;
	CPropListColumns			m_prop_list_columns;
	CPropListQuality			m_prop_list_quality;
	CPropListGeneral			m_prop_list_general;
	DWORD						m_dwCookie;
	BOOL						m_bDoneThorough;
	BOOL						m_bDropMode;

	COLORREF				m_quality_cols[3];
	//application settings
	int						m_frameresolution;
	BOOL					m_readallunknown;
	BOOL					m_readallinitial;
	BOOL					m_monoquality;
	BOOL					m_autosize;

	CPidl					m_lpiFolder;
	//CReadAllDlg
//	HWND					m_hWndRead;

public:

	DWORD					m_idSort;
	BOOL					m_bReverseSort;

	//CFrameWindowStuff
	HWND	CreateImp(HWND hWndParent = NULL, RECT *pPos=NULL);
	
	//Settings stuff
	void			SaveSettings(CIniFile &ini);
	void			LoadSettings(const CIniFile &ini);

	tstring			ExportXML();
	tstring			ExportTextFile();
	void			AddPropPages(CSettingsDlg *psheet);
	void			ApplyPropPageChanges(BOOL dirty);
	void			InitPropPages(BOOL *pDirty);
	
	CEncSpotList(CWindow *pMain) :  m_lptvRecentData(NULL), m_bDropMode(FALSE), m_dwCookie(0), m_idSort(ID_NAME), m_pMain(pMain), m_Header(pMain),
													 m_frameresolution(100), m_thread(TRUE, &m_frameresolution, &m_monoquality, &m_readallunknown){}
	virtual	~CEncSpotList();
	
private:
	void RefreshVisibleStates();
	vector<int>				GetSelectedItems();

	vector<tstring>			GetFolderContents();
	vector<CPidl>			GetFolderPidls();
	void					DoThoroughScan();
	LRESULT					FileOperation(vector<tstring> fnames, BOOL bMove);
	void UpdateProgressStatus(int nSoFar);
	void DoDetails(int i);

	bool			IsMp3(const tstring &path);
	int				FindListViewInsertionPos(LPLVITEMDATA lplvid);
	void			InitColumns();
	tstring			GetSelectedListContents();
	LPLVITEMDATA	AddItemToList(const CPidl &lpi, IShellFolder *pShellFolder, CComPtr<IMalloc> spMalloc);
	BOOL			FillListView(IShellFolder* pShellFolder, BOOL delete_all);
	

	void Sort();
	static int CALLBACK EncSpotCompareProc(LPARAM lparam1, LPARAM lparam2,LPARAM lparamSort);
	
	void UpdateFramesStatus();
	void UpdateDirStatus();
 
	void PostStatusMessage(int panel, tstring msg);
	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnShellOp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnSelectAll(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnDrop(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnBadFile(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnTimer(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnDeleteSelected(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);	
	LRESULT OnFill(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);	
	LRESULT OnThreadDone(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/);
	LRESULT OnRefresh(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/);
	LRESULT OnStop(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/);
	LRESULT OnRename(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/);
	LRESULT OnCustomDraw(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);


	LRESULT OnRClk(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
	LRESULT OnBeginDrag(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
	LRESULT OnItemsChanged(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
	LRESULT OnItemChanged(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	DWORD OnPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW lpNMCustomDraw)		{ return  CDRF_NOTIFYITEMDRAW;   }
	DWORD OnItemPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW lpNMCustomDraw) { return CDRF_NOTIFYSUBITEMDRAW; }
	DWORD OnSubItemPrePaint(int /*idCtrl*/, LPNMCUSTOMDRAW lpNMCustomDraw);

	LRESULT OnLVItemKeyDown(int , LPNMHDR pnmh, BOOL& bHandled);
	LRESULT OnLVDblClk(int , LPNMHDR pnmh, BOOL& );
	LRESULT OnEndLabelEdit(int , LPNMHDR pnmh, BOOL& );
	LRESULT OnBeginLabelEdit(int , LPNMHDR pnmh, BOOL& );
	LRESULT OnColumnClick(int , LPNMHDR pnmh, BOOL& );
	LRESULT OnGetDispInfo(int , LPNMHDR pnmh, BOOL& );

	LRESULT OnExport(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);


public:
	

	DECLARE_WND_SUPERCLASS(NULL, CEncSpotList::GetWndClassName())

		BEGIN_MSG_MAP(CEncSpotList)

		MESSAGE_HANDLER(WM_CREATE,							OnCreate)
		MESSAGE_HANDLER(WM_DELETE,							OnDeleteSelected)
		MESSAGE_HANDLER(WM_THREADDONE,						OnThreadDone)
		MESSAGE_HANDLER(WM_BADFILE,							OnBadFile)
		MESSAGE_HANDLER(WM_REFRESH,							OnRefresh)
		MESSAGE_HANDLER(WM_TIMER,							OnTimer)
		MESSAGE_HANDLER(WM_RENAME,							OnRename)
		MESSAGE_HANDLER(WM_STOP,							OnStop)
		MESSAGE_HANDLER(WM_EXPORT,							OnExport)
		MESSAGE_HANDLER(WM_DROP,							OnDrop)
		MESSAGE_HANDLER(WM_SHELLOP,							OnShellOp)
		MESSAGE_HANDLER(WM_SELECTALL,						OnSelectAll)


		MESSAGE_HANDLER(WM_DESTROY,							OnDestroy)
		MESSAGE_HANDLER(WM_FILL,							OnFill)
		
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_GETDISPINFO,		OnGetDispInfo)
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_ENDLABELEDIT,		OnEndLabelEdit)
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_BEGINLABELEDIT,	OnBeginLabelEdit)
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_COLUMNCLICK,		OnColumnClick)
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_BEGINDRAG,		OnBeginDrag)
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_KEYDOWN,			OnLVItemKeyDown		)
		REFLECTED_NOTIFY_CODE_HANDLER(NM_RCLICK ,			OnRClk)	
		REFLECTED_NOTIFY_CODE_HANDLER(NM_DBLCLK ,			OnLVDblClk		)
		REFLECTED_NOTIFY_CODE_HANDLER(NM_CUSTOMDRAW,		OnCustomDraw)
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_ODSTATECHANGED,	OnItemsChanged)
		REFLECTED_NOTIFY_CODE_HANDLER(LVN_ITEMCHANGED,		OnItemChanged)
		
		DEFAULT_REFLECTION_HANDLER( )
		
	END_MSG_MAP()

	BEGIN_LIST_DATA_MAP(CEncSpotList, LVITEMDATA)
		LIST_DATA_TSTRING			(ID_NAME	,	_T("Name")				,name	,		LVSCW_AUTOSIZE)
		LIST_DATA_TSTRING			(ID_ENCODER	,	_T("Encoder (guess)")   ,data_string.encoder,		100)
		LIST_DATA_TSTRING			(ID_MODE	,	_T("Stereo Mode")	,	,data_string.mode		,	80)
		LIST_DATA_TSTRING			(ID_CUT		,	_T("Complete")			,data_string.complete,		50)
		LIST_DATA_TSTRING			(ID_TYPE	,	_T("Type")				,data_string.type	,		100)
		
		LIST_DATA_TSTRING_SPECIALSORT(ID_SIZE	,	_T("Size (bytes)")	,	data_string.fsize,			data.fsize,			100)
		LIST_DATA_TSTRING_SPECIALSORT(ID_QUALITY,	_T("Quality")		,	null	  ,					data_string.quality,			50)
		LIST_DATA_TSTRING_SPECIALSORT(ID_BITRATE,	_T("Bitrate (kbps)")    ,data_string.bitrate,		data.bitrate,			60)
		LIST_DATA_TSTRING_SPECIALSORT(ID_VBR,		_T("Vbr")				,data_string.vbr,			data.vbr,			40)
		LIST_DATA_TSTRING_SPECIALSORT(ID_CUE,		_T("Cue Sheet")			,data_string.cue,			data.cue,			40)
		LIST_DATA_TSTRING_SPECIALSORT(ID_FREQUENCY,	_T("Freq. (Hz)")		,data_string.freq	,		data.base_freq,		70)
		LIST_DATA_TSTRING_SPECIALSORT(ID_FRAMES	,	_T("Frames")			,data_string.frames	,		data.frameCount	,	50)
		LIST_DATA_TSTRING_SPECIALSORT(ID_LENGTH	,	_T("Length (h:m:s)")	,data_string.length	,		data.length	,		100)
		LIST_DATA_TSTRING_SPECIALSORT(ID_RESERVOIR,	_T("Max. Reservoir")	,data_string.max_reservoir,	data.max_reservoir, 80)	

	END_LIST_DATA_MAP()			


};



#endif // !defined(AFX_ENCSPOTLIST_H__70C7F02F_06BB_11D5_A176_080046068885__INCLUDED_)
