
#ifndef DATAOBJ_H
#define DATAOBJ_H


#define MAX_NUM_FORMAT 5




typedef CComEnum<IEnumFORMATETC, &IID_IEnumFORMATETC,FORMATETC, _Copy<FORMATETC> > 
		CEnumFormatEtc;

/**************************************************************************
   class definitions
**************************************************************************/

class CDataObject: public IDataObjectImpl<CDataObject>, public CComObjectRoot
{
private:
   LPFORMATETC m_pFormatEtc;
   LPSTGMEDIUM m_pStgMedium;
   DWORD m_cDataCount;

public:

	IDataAdviseHolder  *m_spDataAdviseHolder;

	BEGIN_COM_MAP(CDataObject)
		COM_INTERFACE_ENTRY(::IDataObject)
	END_COM_MAP()

	void Setup(CLIPFORMAT cf, tstring data, BOOL bUseCopy = FALSE, BOOL bCopy = FALSE)
	{
		FORMATETC   fe;
		fe.cfFormat = CF_HDROP;
		fe.ptd = NULL;
		fe.dwAspect = DVASPECT_CONTENT;
		fe.lindex = -1;
		fe.tymed = TYMED_HGLOBAL;

		DROPFILES drop;
		memset(&drop, 0, sizeof(DROPFILES));
		drop.pFiles = sizeof(DROPFILES);

		HGLOBAL hg;
		hg = GlobalAlloc(GMEM_FIXED, sizeof(DROPFILES) + data.size());
		GlobalLock(hg);
		BYTE *pGlobal = reinterpret_cast<BYTE *>(hg);

		memcpy(pGlobal, &drop, sizeof(drop));
		memcpy(pGlobal+sizeof(DROPFILES), data.data(), data.size());
		
		GlobalUnlock(hg);

		STGMEDIUM   sm;
		sm.tymed = TYMED_HGLOBAL;
		sm.pUnkForRelease = NULL;
		sm.hGlobal = hg;

		SetData(&fe, &sm, TRUE);

		if (bUseCopy)
		{
			FORMATETC   fe;
			fe.cfFormat = cf;
			fe.ptd = NULL;
			fe.dwAspect = DVASPECT_CONTENT;
			fe.lindex = -1;
			fe.tymed = TYMED_HGLOBAL;

			DROPFILES drop;
			memset(&drop, 0, sizeof(DROPFILES));
			drop.pFiles = sizeof(DROPFILES);

			HGLOBAL hg;
			hg = GlobalAlloc(GMEM_FIXED, sizeof(DWORD));
			GlobalLock(hg);
			DWORD *pGlobal = reinterpret_cast<DWORD *>(hg);
			*pGlobal = bCopy?DROPEFFECT_COPY:DROPEFFECT_MOVE;	
			GlobalUnlock(hg);

			STGMEDIUM   sm;
			sm.tymed = TYMED_HGLOBAL;
			sm.pUnkForRelease = NULL;
			sm.hGlobal = hg;

			SetData(&fe, &sm, TRUE);

		}
	}
		
	LPTSTR GetNextName(LPCTSTR pszFiles)
	{
		LPTSTR   pszTemp = (LPTSTR)pszFiles;
		while(*pszTemp)
		   {
		   pszTemp++;
		   }

		pszTemp++;
 
		return *pszTemp ? pszTemp : NULL;
	}

   
	CDataObject()
	{
		m_pFormatEtc = NULL;
		m_pStgMedium = NULL;
		m_cDataCount = 0;
		m_spDataAdviseHolder = NULL;
	}
   
	virtual CDataObject::~CDataObject()
	{
		if(m_pFormatEtc)
			delete m_pFormatEtc;

		if(m_pStgMedium)
		{
			DWORD i;

			for(i = 0; i < m_cDataCount; i++)
			  ReleaseStgMedium(&m_pStgMedium[i]);

			delete m_pStgMedium;
		}
	}

 

   /* IDataObject methods */    
   STDMETHOD(IDataObject_GetData)(LPFORMATETC pFE,  LPSTGMEDIUM pSM)
   {
		if(pFE == NULL || pSM == NULL)
		   return E_INVALIDARG;

		ZeroMemory(pSM, sizeof(STGMEDIUM));

		DWORD i;

		for(i = 0; i < m_cDataCount; i++)
		{
			if(   (pFE->tymed & m_pFormatEtc[i].tymed) && 
				 (pFE->dwAspect == m_pFormatEtc[i].dwAspect) && 
				 (pFE->cfFormat == m_pFormatEtc[i].cfFormat) &&
				 (pFE->lindex == m_pFormatEtc[i].lindex))
			{
				HRESULT  hr;

				hr = CopyStgMedium(&m_pStgMedium[i], pSM);

				if(pSM->tymed == TYMED_HGLOBAL)
				{
				 //this tell's the caller not to free the global memory
				 QueryInterface(IID_IUnknown, (LPVOID*)&pSM->pUnkForRelease);
				}

				return hr;
			}
		}

		return DV_E_FORMATETC;
   }


   STDMETHOD(QueryGetData)(LPFORMATETC pFE)
   {
		if(!pFE)
			return DV_E_FORMATETC;

		if(!(DVASPECT_CONTENT & pFE->dwAspect))
			return DV_E_DVASPECT;

		//now check for an appropriate TYMED.
		DWORD    i;
		HRESULT  hr = DV_E_TYMED;

		for(i = 0; i < m_cDataCount; i++)
		{
			if(pFE->tymed & m_pFormatEtc[i].tymed)
			{
				if(pFE->cfFormat == m_pFormatEtc[i].cfFormat)
				 return S_OK;
				else
				 hr = DV_E_CLIPFORMAT;
			}
			else
				hr = DV_E_TYMED;
		}

		return hr;
   }


   STDMETHOD(GetCanonicalFormatEtc)(LPFORMATETC, LPFORMATETC) { return DATA_S_SAMEFORMATETC; }


   STDMETHOD(SetData)(LPFORMATETC pFE, LPSTGMEDIUM pSM, BOOL fRelease)
   {
		LPFORMATETC pfeNew;
		LPSTGMEDIUM psmNew;

		m_cDataCount++;

		pfeNew = new FORMATETC[m_cDataCount];
		psmNew = new STGMEDIUM[m_cDataCount];

		if(pfeNew && psmNew)
		{
		   ZeroMemory(pfeNew, sizeof(FORMATETC) * m_cDataCount);
		   ZeroMemory(psmNew, sizeof(STGMEDIUM) * m_cDataCount);
   
		   //copy the existing data
		   if(m_pFormatEtc)
			  CopyMemory(pfeNew, m_pFormatEtc, sizeof(FORMATETC) * (m_cDataCount - 1));
			 
		   if(m_pStgMedium)
			  CopyMemory(psmNew, m_pStgMedium, sizeof(STGMEDIUM) * (m_cDataCount - 1));
			 
		   //add the new data
		   pfeNew[m_cDataCount - 1] = *pFE;
		   if(fRelease)
			  psmNew[m_cDataCount - 1] = *pSM;
		   else
			  CopyStgMedium(pSM, &psmNew[m_cDataCount - 1]);

		   LPFORMATETC pfeTemp = m_pFormatEtc;
		   m_pFormatEtc = pfeNew;
		   pfeNew = pfeTemp;

		   LPSTGMEDIUM psmTemp = m_pStgMedium;
		   m_pStgMedium = psmNew;
		   psmNew = psmTemp;
		}

		if(pfeNew)
		   delete pfeNew;

		if(psmNew)
		   delete psmNew;

		if(m_pFormatEtc && m_pStgMedium)
		   return S_OK;

		return E_OUTOFMEMORY;
   }



   STDMETHOD(EnumFormatEtc)(DWORD dwDir, LPENUMFORMATETC* ppEnum)
   {
		*ppEnum = NULL;

		switch(dwDir)
		   {
		   case DATADIR_GET:
			  {
				CComObject<CEnumFormatEtc> *pEnum = NULL;
				CComObject<CEnumFormatEtc>::CreateInstance(&pEnum);
				pEnum->Init(m_pFormatEtc, &m_pFormatEtc[m_cDataCount],NULL, AtlFlagCopy);
				pEnum->QueryInterface(ppEnum);
				if(*ppEnum)
					return S_OK;
			  break;
			  }
   
		   default:
			  return OLE_S_USEREG;
		   }

		return E_OUTOFMEMORY;
   }

};

#endif   //DATAOBJ_H
