// Pidl.h: interface for the CPidl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PIDL_H__6A9E8100_01A2_11D5_A176_080046068885__INCLUDED_)
#define AFX_PIDL_H__6A9E8100_01A2_11D5_A176_080046068885__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "EncSpot.h"

#include <shlobj.H>

class CPidl  
{

public:
	LPITEMIDLIST m_pidl;


	CPidl() : m_pidl(NULL){}
	void Free() { if (m_pidl) CoTaskMemFree(m_pidl); m_pidl = NULL; }

	LPITEMIDLIST operator=(LPITEMIDLIST in) { m_pidl = in; return in; }

	CPidl(LPITEMIDLIST pidl) { m_pidl = pidl; }

	LPITEMIDLIST Copy(LPCITEMIDLIST lpi,IMalloc* lpMalloc = NULL)
	{
		IMalloc *lpMalloc2 = lpMalloc;
		if (lpMalloc == NULL)
		{
			HRESULT hr=SHGetMalloc(&lpMalloc2);
			if (FAILED(hr)) return 0;
		}

		LPITEMIDLIST lpiTemp = 0;

	   lpiTemp=(LPITEMIDLIST)lpMalloc2->Alloc(lpi->mkid.cb+sizeof(lpi->mkid.cb));

	   CopyMemory((PVOID)lpiTemp, (CONST VOID *)lpi, lpi->mkid.cb+sizeof(lpi->mkid.cb));

	   m_pidl = lpiTemp;
	   
	   if (!lpMalloc)
		   lpMalloc2->Release();

	   return lpiTemp;
	}



		
	UINT size() const
	{
		CPidl pidl = m_pidl;
		UINT cbTotal = 0;
		if (pidl.m_pidl)
		{
			cbTotal += sizeof(pidl.m_pidl->mkid.cb);       // Null terminator
			while (pidl.m_pidl->mkid.cb)
			{
				cbTotal += pidl.m_pidl->mkid.cb;
				pidl = pidl.Next();
			}
		}

		return cbTotal;
	}

	int length() const
	{
		int len = 0;
		CPidl pidl = m_pidl;
		while (pidl.m_pidl->mkid.cb)
		{
			pidl = pidl.Next();
			len++;
		}

		return len;
	}

	BOOL CreateRelativePidl(const LPITEMIDLIST &pChildPidl, IMalloc *lpMalloc)
	{
		CPidl pTempPidl = pChildPidl;
		if (pTempPidl.length() == 0)
			return FALSE;

		int len = pTempPidl.length();
		for (int i = 0;i<(len-1);i++)
			pTempPidl = pTempPidl.Next();

		Copy(pTempPidl,lpMalloc);




		return TRUE;
	}

	BOOL CreateParentPidl(const LPITEMIDLIST &pChildPidl, IMalloc *lpMalloc)
	{
		CPidl pTempPidl = pChildPidl;
		if (pTempPidl.length() == 0)
			return FALSE;

		int len = pTempPidl.length();

		//find size of sub-pidl
		CPidl pidl = pTempPidl;
		UINT cbTotal = 0;

		for (int i = 0;i<(len-1);i++, pidl = pidl.Next())
			cbTotal += pidl.m_pidl->mkid.cb;
	
		LPITEMIDLIST lpiTemp=(LPITEMIDLIST)lpMalloc->Alloc(cbTotal + sizeof(pChildPidl->mkid.cb));
		memset(lpiTemp, 0, cbTotal + sizeof(pChildPidl->mkid.cb));

		CopyMemory((PVOID)lpiTemp, (CONST VOID *)pChildPidl, cbTotal);
		m_pidl = lpiTemp;

		return TRUE;
	}

	operator LPITEMIDLIST() { return m_pidl; }

	operator LPCITEMIDLIST() const { return m_pidl; }

	LPITEMIDLIST Next()
	{
	   LPSTR lpMem=(LPSTR)m_pidl;

	   lpMem+=m_pidl->mkid.cb;

	   return (LPITEMIDLIST)lpMem;
	}

	CPidl Append(const CPidl other,IMalloc* lpMalloc) const
	{
		LPITEMIDLIST pidlNew;
		UINT cb1 = 0;
		UINT cb2 = 0;

		if (m_pidl)  //May be NULL
		   cb1 = size() - sizeof(m_pidl->mkid.cb);
		else
		   cb1 = 0;

		cb2 = other.size();

		pidlNew = Allocate(cb1 + cb2,lpMalloc);
		if (pidlNew)
		{
			if (m_pidl)
			   memcpy(pidlNew, m_pidl, cb1);

			memcpy(((LPSTR)pidlNew) + cb1, other.m_pidl, cb2);
		}

		return pidlNew;
	}

	


	int GetIcon(UINT uFlags)
	{
		SHFILEINFO    sfi;
		memset(&sfi, 0, sizeof(SHFILEINFO));

		SHGetFileInfo((LPTSTR)m_pidl, 
					 0,
					 &sfi, 
					 sizeof(SHFILEINFO), 
					 uFlags);

		return sfi.iIcon;
	}

	BOOL Exists(LPSHELLFOLDER lpsf)
	{
		TCHAR name[MAX_PATH];
		return GetName(lpsf, SHGDN_NORMAL, name);
	}


		
	LPITEMIDLIST GetFullyQualPidl(LPSHELLFOLDER lpsf) const 
	{

		USES_CONVERSION;


	   TCHAR szBuff[MAX_PATH];
	   
	   CComPtr<IShellFolder> spDeskTop;
	   LPITEMIDLIST  lpifq;
	   ULONG ulEaten, ulAttribs;
	   HRESULT hr;

	   if (!GetName(lpsf, SHGDN_FORPARSING, szBuff))
		  return NULL;

	   hr=SHGetDesktopFolder(&spDeskTop);

	   if (FAILED(hr))  return NULL;

	   
	   hr=spDeskTop->ParseDisplayName(NULL,
										NULL,
										T2W(szBuff),
										&ulEaten,
										&lpifq,
										&ulAttribs);

   
	   if (FAILED(hr))   return NULL;

	   return lpifq;
	}



	tstring GetPath(LPSHELLFOLDER lpsf) const
	{
		TCHAR path[MAX_PATH];
		path[0] = 0;
		GetName(lpsf, SHGDN_NORMAL | SHGDN_FORPARSING, path);
		return path;
	}

	tstring GetName(LPSHELLFOLDER lpsf) const
	{
		TCHAR name[MAX_PATH];
		name[0] = 0;
		GetName(lpsf, SHGDN_NORMAL , name);
		return name;
	}



		

	BOOL GetName (LPSHELLFOLDER lpsf, DWORD dwFlags, LPTSTR lpFriendlyName) const
	{

		USES_CONVERSION;

		BOOL   bSuccess=TRUE;
		STRRET str;

		if (NOERROR==lpsf->GetDisplayNameOf(m_pidl,dwFlags, &str))
		{
		  switch (str.uType)
		  {
			 case STRRET_WSTR:
				 lstrcpy(lpFriendlyName, W2T(str.pOleStr));
				 break;

			 case STRRET_OFFSET:
				 lstrcpy(lpFriendlyName, A2T((LPSTR)m_pidl+str.uOffset));
				 break;

			 case STRRET_CSTR:             
				 lstrcpy(lpFriendlyName, A2T(str.cStr));
				 break;

			 default:
				 bSuccess = FALSE;
				 break;
		  }
		}
		else
		  bSuccess = FALSE;

		return bSuccess;
	}



private:

	
	



	static LPITEMIDLIST Allocate(UINT cbSize,IMalloc* lpMalloc)
	{
		IMalloc *lpMalloc2 = lpMalloc;
		if (lpMalloc == NULL)
		{
			HRESULT hr=SHGetMalloc(&lpMalloc2);
			if (FAILED(hr)) return 0;
		}

		LPITEMIDLIST pidl=0;
		pidl = (LPITEMIDLIST)lpMalloc2->Alloc(cbSize);

		if (pidl)
			memset(pidl, 0, cbSize);      // zero-init for external task   

		if (!lpMalloc)
			lpMalloc2->Release();
		return pidl;
	}


};

#endif // !defined(AFX_PIDL_H__6A9E8100_01A2_11D5_A176_080046068885__INCLUDED_)
