// EncSpotTree.cpp: implementation of the CEncSpotTree class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <atlwin.h>
#include <atlddx.h>
#include <atldlgs.h>
#include <atlctrls.h>
#include <atlctrlx.h>
#include <atlmisc.h>
#include <shlobj.h>

#include "resource.h"
#include "PropTree.h"
#include "EncSpotTree.h"
#include "itemdata.h"


#ifdef _DEBUG
	int TVITEMDATA::ref_debug = 0;
#endif

extern tstring g_cmdline;




//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEncSpotTree::CEncSpotTree(CWindow *pParent) : m_pMain(pParent), m_bQuietSelect(FALSE)
{

}

CEncSpotTree::~CEncSpotTree()
{
	int destructor_called = 1;

}

void CEncSpotTree::SaveSettings(CIniFile &ini)
{
	ini.SetString(_T("tree_settings"),_T("initdir"),m_default_dir);
	ini.SetInt(_T("tree_settings"),_T("userecent"),m_userecent);
}
void CEncSpotTree::LoadSettings(const CIniFile &ini)
{
	m_default_dir	 = ini.GetString(_T("tree_settings"),_T("initdir"),_T(""));
	if (m_default_dir  == _T(""))
		m_default_dir = get_desktop_path();

	m_userecent		 = ini.GetInt(_T("tree_settings"),_T("userecent"),0);

}


HWND CEncSpotTree::CreateImp(HWND hWndParent, RECT *pPos)
{

	Create(hWndParent, *pPos, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | 
									TVS_HASLINES |TVS_LINESATROOT | TVS_HASBUTTONS  | TVS_SHOWSELALWAYS, 0);

	InitTreeView();
	return m_hWnd;

}

LRESULT CEncSpotTree::OnItemExpanding(int , LPNMHDR pnmh, BOOL& )
{
	LOG(_T("CEncSpotTree::OnItemExpanding"));

	CWaitCursor wait;
	LPNMTREEVIEW pnmtv = (LPNMTREEVIEW)pnmh;
	if ((pnmtv->itemNew.state & TVIS_EXPANDEDONCE))
		 return 0L;

	return ExpandItem(&pnmtv->itemNew);
}



LRESULT CEncSpotTree::OnDeleteItem(int , LPNMHDR pnmh, BOOL& )
{

	LPNMTREEVIEW pnmtv = (LPNMTREEVIEW)pnmh;
	LPTVITEMDATA lptvid	=	(LPTVITEMDATA)pnmtv->itemOld.lParam;
	delete lptvid;  

	return 0;
}


void CEncSpotTree::InitTreeView()
{

	IShellFolder *spFolder = NULL;

	HRESULT hr = SHGetDesktopFolder(&spFolder);
	if(FAILED(hr)) return ;

	//LPITEMIDLIST pDeskIdl;
	CPidl pDeskIdl;
	SHGetSpecialFolderLocation(m_hWnd,CSIDL_DESKTOP,&pDeskIdl.m_pidl);
		
	SHFILEINFO sfi;
	memset(&sfi, 0, sizeof(SHFILEINFO));
	HIMAGELIST hImageList = (HIMAGELIST)SHGetFileInfo(_T("C:\\"), 
									  0,
									   &sfi, 
									   sizeof(SHFILEINFO), 
									   SHGFI_SYSICONINDEX | SHGFI_SMALLICON);

	// Attach ImageList to TreeView
	if (hImageList && IsWindow())
		SetImageList(hImageList,0);

	CWaitCursor wait;  


	//add desktop
	CComPtr<IMalloc>        spMalloc;
	hr=SHGetMalloc(&spMalloc);

	if (FAILED(AddItemToTree(SFGAO_HASSUBFOLDER | SFGAO_FOLDER, spFolder, pDeskIdl,spMalloc,NULL,TVI_ROOT)))
		return;

	Expand(GetRootItem());

	if (!g_cmdline.empty())
	{
		if (g_cmdline[0]=='"')
			g_cmdline.erase(g_cmdline.begin());
		if (*(g_cmdline.end()-1)=='"')
			g_cmdline.erase(g_cmdline.end()-1);

		CString szExt = PathFindExtension(g_cmdline.c_str());
		szExt.MakeLower();
		if (szExt==".mp3")
		{
			m_bQuietSelect = TRUE;
			BeginExpand(get_desktop_path(), get_desktop_path());

			vector<tstring> *pList = new vector<tstring>;
			pList->push_back(g_cmdline);
			m_pMain->PostMessage(WM_DROP, NULL, (LPARAM)pList);
		}
		else
			BeginExpand(get_desktop_path(), g_cmdline);

	}
	else
		BeginExpand(get_desktop_path(), m_default_dir);

	pDeskIdl.Free();

	return ;
}


HRESULT CEncSpotTree::AddItemToTree(ULONG ulAttrs, 				
									IShellFolder *pShellFolder, 
									CPidl lpi, 
									CComPtr<IMalloc> &spMalloc,
									CPidl lpifq, 
									HTREEITEM hParent)
{
	HTREEITEM   hPrev		= NULL;                 

	
	TVITEM tvi;
	TV_INSERTSTRUCT tvins;                       

	tvi.mask  = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE |TVIF_PARAM;
	if (ulAttrs & SFGAO_HASSUBFOLDER)
	{
		tvi.cChildren=1;
		tvi.mask |= TVIF_CHILDREN;
	}

	LPTVITEMDATA lptvid = new TVITEMDATA;
	if (!lptvid)
	{
		lpi.Free();
		return E_FAIL;
	}

	lptvid->root =  (hParent == TVI_ROOT);


	TCHAR  szBuff[256];
	if (!lpi.GetName(pShellFolder, SHGDN_NORMAL, szBuff))
	{
		lpi.Free();
		return E_FAIL;
	}

	tvi.pszText    = szBuff;
	tvi.cchTextMax = MAX_PATH;

	CPidl lpifqThisItem = NULL;
	lpifqThisItem=lpifq.Append(lpi,spMalloc);
	lptvid->lpi.Copy(lpi,spMalloc);

	tvi.iImage = lpifqThisItem.GetIcon(SHGFI_PIDL | 
                              SHGFI_SYSICONINDEX | 
                              SHGFI_SMALLICON);
   
	tvi.iSelectedImage = lpifqThisItem.GetIcon( SHGFI_PIDL | 
									  SHGFI_SYSICONINDEX | 
									  SHGFI_SMALLICON |
									  SHGFI_OPENICON);

	lptvid->spParentFolder.p=pShellFolder;    //Store the parent folders SF
	pShellFolder->AddRef();


	lptvid->lpifq=lpifq.Append(lpi,spMalloc);

	tvi.lParam = (LPARAM)lptvid;

	tvins.item         = tvi;
	tvins.hInsertAfter = hPrev;
	tvins.hParent      = hParent;

	// Add the item to the tree and combo
	hPrev = InsertItem(&tvins);

	lpifqThisItem.Free();

	return S_OK;
}




void CEncSpotTree::BeginExpand(tstring desktop, tstring defaultdir)
{

	USES_CONVERSION;

	CComPtr<IShellFolder> spFolder;

	HRESULT hr = SHGetDesktopFolder(&spFolder);
	if (FAILED(hr))
		return;

	CString str = defaultdir.c_str();
	str.MakeLower();
	tstring part = LPCTSTR(str);
	CString root0= desktop.c_str();
	root0.MakeLower();
	tstring root  = LPCTSTR(root0);
	
	if (part == root)
	{
		SelectItem(GetRootItem());
		return;
	}

	int pos = part.find(root);
	if (pos != tstring::npos)
		part = part.substr(pos + root.size()+1,tstring::npos);


	
	CPidl	fullpidl;
	ULONG	parsed = 0;
	spFolder->ParseDisplayName(m_hWnd,NULL,T2W(part.c_str()),&parsed, &fullpidl.m_pidl, NULL);
	if (fullpidl.m_pidl == NULL)
	{
	/*	tstring message = _T("Unable to locate folder \r\n") + part;
		MessageBox(message.c_str(), _T("Uh-oh"));*/
		SelectItem(GetRootItem());
		return;
	}
		


	HTREEITEM hFinal = ExpandToPath(GetRootItem(), fullpidl);
	if (hFinal)
		SelectItem(hFinal);
	fullpidl.Free();
}



//recursive
HTREEITEM CEncSpotTree::ExpandToPath(HTREEITEM hItem, CPidl lpi)
{
	if (lpi.m_pidl==0)
		return NULL;

	int size = lpi.size();
	if (size == sizeof(lpi.m_pidl->mkid.cb))
		return hItem;

	HTREEITEM hChild = GetChildItem(hItem);
	while (hChild)
	{
		LPTVITEMDATA lptvid  = reinterpret_cast<LPTVITEMDATA>(GetItemData(hChild));
	
		if (lptvid->lpi.m_pidl->mkid.cb != lpi.m_pidl->mkid.cb)
		{
			hChild = GetNextSiblingItem(hChild);
			continue;
		}

		if (0==memcmp(lptvid->lpi.m_pidl->mkid.abID, lpi.m_pidl->mkid.abID,lpi.m_pidl->mkid.cb - sizeof(USHORT)))
			break;

		hChild = GetNextSiblingItem(hChild);
	}

	if (!hChild)
		return NULL;

	if (GetChildItem(hChild) == NULL)
		Expand(hChild);

	return ExpandToPath(hChild, lpi.Next());

}


HRESULT CEncSpotTree::FillTreeView(IShellFolder * pShellFolder,LPITEMIDLIST  lpifq,HTREEITEM hParent)
{
	if(!pShellFolder) return E_POINTER;
	
	CComPtr<IMalloc>        spMalloc;
	HRESULT hr=SHGetMalloc(&spMalloc);
	if (FAILED(hr))  return hr;
	
	CComPtr<IEnumIDList>    spIDList;
	hr=	pShellFolder->EnumObjects(	GetParent(),
									SHCONTF_FOLDERS | SHCONTF_NONFOLDERS,&spIDList);
	if (FAILED(hr))  return hr;


	CPidl    lpi;
	
	ULONG		nCounter	= 0;
	ULONG		ulFetched	= 0;
	
	// Hourglass on!
	CWaitCursor wait;
	while (S_OK==spIDList->Next(1,&lpi.m_pidl,&ulFetched))
	{
		                       
		
		ULONG ulAttrs = SFGAO_HASSUBFOLDER | SFGAO_FOLDER;

		hr = pShellFolder->GetAttributesOf(1, (LPCITEMIDLIST*)&lpi.m_pidl, &ulAttrs);

		if(	SUCCEEDED(hr) && ( ulAttrs & (SFGAO_HASSUBFOLDER | SFGAO_FOLDER) )  )
		{
			nCounter++;
			if (ulAttrs & SFGAO_FOLDER)
				if (!SUCCEEDED(hr = AddItemToTree(ulAttrs, pShellFolder, lpi.m_pidl,spMalloc,lpifq,hParent)))
					return hr;
		
			
		}//if succeeded
		lpi.Free();
	}
	
	if(!nCounter)
	{
		TVITEM tvi;
		tvi.hItem = hParent;
		tvi.cChildren = 0;
		tvi.mask = TVIF_CHILDREN;
		SetItem(&tvi);
	}

	return hr;
}



int CALLBACK CEncSpotTree::TreeViewCompareProc(LPARAM lparam1,LPARAM lparam2,LPARAM lparamSort)
{
	LPTVITEMDATA lptvid1	=(LPTVITEMDATA)lparam1;
	if(!lptvid1 || !lptvid1->spParentFolder) return -1;

	LPTVITEMDATA lptvid2	=(LPTVITEMDATA)lparam2;
	if(!lptvid2 || !lptvid2->spParentFolder) return -1;

	HRESULT hr = lptvid1->spParentFolder->CompareIDs(0,lptvid1->lpi,lptvid2->lpi);
	if (FAILED(hr)) return 0;

	return (short)SCODE_CODE(GetScode(hr));
}



LRESULT CEncSpotTree::ExpandItem(LPTVITEM lptvi)
{



	LPTVITEMDATA   lptvid=(LPTVITEMDATA)lptvi->lParam;


	HRESULT hr;
	IShellFolder *spFolder = NULL;
	if (lptvid->root)
		spFolder = lptvid->spParentFolder;
	else
	{
		hr=lptvid->spParentFolder->BindToObject(lptvid->lpi,0,IID_IShellFolder,(LPVOID *)&spFolder);
		if(FAILED(hr)) return hr;
	}
	
	FillTreeView(spFolder,lptvid->lpifq,lptvi->hItem);
	
	TV_SORTCB tvscb;
	tvscb.hParent     = lptvi->hItem;
	tvscb.lpfnCompare = TreeViewCompareProc;
	tvscb.lParam      = 0;

	SortChildrenCB(&tvscb);

	if (!lptvid->root)
		spFolder->Release();

	return 0L;

}

LRESULT CEncSpotTree::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{

	HTREEITEM hItem = GetSelectedItem();
	LPTVITEMDATA lptvid = reinterpret_cast<LPTVITEMDATA>(GetItemData(hItem));
	if (!hItem)
		return 0;

	if (m_userecent)
	{
		if (lptvid->root)
			m_default_dir = get_desktop_path();
		else
			m_default_dir = lptvid->lpi.GetPath(lptvid->spParentFolder);
	}

	bHandled = FALSE;
	return 0;
}

void CEncSpotTree::InitPropPages(BOOL *pDirty)
{
	lstrcpy(m_proptree.m_initdir, m_default_dir.c_str());
	m_proptree.m_userecent = m_userecent;
	m_proptree.m_pDirty = pDirty;
}

void CEncSpotTree::ApplyPropPageChanges(BOOL dirty)
{
	m_default_dir	= m_proptree.m_initdir;
	m_userecent		= m_proptree.m_userecent;
}

void CEncSpotTree::AddPropPages(CSettingsDlg *psheet)
{
	psheet->AddPage(m_proptree);
}


LRESULT CEncSpotTree::OnRefresh(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	//get desktop
	IShellFolder *spFolder = NULL;
	if(FAILED(SHGetDesktopFolder(&spFolder))) return 0;

	CComPtr<IMalloc>        spMalloc;
	if (FAILED(SHGetMalloc(&spMalloc)))
	{
		spFolder->Release();
		return 0;
	}
	CPidl pDeskIdl;
	SHGetSpecialFolderLocation(m_hWnd,CSIDL_DESKTOP,&pDeskIdl.m_pidl);


	tstring old_dir;
	HTREEITEM hItem = GetSelectedItem(); 
	LPTVITEMDATA lptvid = reinterpret_cast<LPTVITEMDATA>(GetItemData(hItem));
		


	if (hItem==NULL)
		old_dir = get_desktop_path();
	else
		old_dir = lptvid->lpi.GetPath(lptvid->spParentFolder);

	DeleteAllItems();


	CWaitCursor wait;  

	

	if (FAILED(AddItemToTree(SFGAO_HASSUBFOLDER | SFGAO_FOLDER, spFolder, pDeskIdl,spMalloc,NULL,TVI_ROOT)))
		return 0;


	Expand(GetRootItem());
	BeginExpand(get_desktop_path(), old_dir);

	pDeskIdl.Free();
	
	return 0;
}

LRESULT CEncSpotTree::OnRClk(int , LPNMHDR pnmh, BOOL& )
{
	
	POINT pt;
	GetCursorPos(&pt);

	POINT pt2 = pt;
	ScreenToClient(&pt2);

	UINT flags;
	HTREEITEM hItem = HitTest(pt2, &flags);
	if (hItem==NULL)
		return 0;

	CMenu menu_parent;
	menu_parent.LoadMenu(IDR_TVIEWPOPUP);
	CMenuHandle menu = menu_parent.GetSubMenu(0);	
	
	menu.SetMenuDefaultItem(0x7FFF);

	
	UINT nCmd = TrackPopupMenu (menu, TPM_LEFTALIGN | TPM_LEFTBUTTON |
                             TPM_RIGHTBUTTON | TPM_RETURNCMD, pt.x, pt.y, 0, m_hWnd, NULL);

	switch(nCmd)
	{
	case 0x7FFF:
		SelectItem(hItem);
		break;	
	case 0x8000:
		m_bQuietSelect = TRUE;
		SelectItem(hItem);
		FillRecursive(hItem);
		
		break;

	}

	return 0;
}




LRESULT CEncSpotTree::OnSelChanged(int , LPNMHDR pnmh, BOOL& )
{
	if (m_bQuietSelect)
	{
		m_bQuietSelect = FALSE;
		return 0;
	}


	LOG(_T("CEncSpotTree::OnSelChanged"));

	LPNMTREEVIEW lpTV = (LPNMTREEVIEW)pnmh;

	WPARAM wParam = lpTV->itemNew.lParam;
	m_pMain->SendMessage(WM_FILLLVIEW,wParam,TRUE);
	return 0L;
} 

//fill list with hItem, then also all of hItem's children.
//remember to fill tview based on hItem as well.
void CEncSpotTree::FillRecursive(HTREEITEM hItem)
{
	LPLVITEMDATA lptvi = reinterpret_cast<LPLVITEMDATA>(GetItemData(hItem));
	WPARAM wParam = reinterpret_cast<WPARAM>(lptvi);
	m_pMain->SendMessage(WM_FILLLVIEW,wParam,FALSE);
}
