// Histogram.cpp: implementation of the CHistogram class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <atlwin.h>
#include <atlframe.h>


#include "Histogram.h"
#include <algorithm>

#define MAX_COLUMNS 20

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHistogram::CHistogram(int ac) : m_acc(ac)
{

	m_colors.push_back(RGB(52,114,212));
	m_colors.push_back(RGB(81,150,247));

}

CHistogram::~CHistogram()
{

}

LRESULT CHistogram::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

LRESULT CHistogram::OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
//	return 0;
	const int bottom_margin = 25;

	CPaintDC dc(m_hWnd);
	int olddc = dc.SaveDC();
	int i;


	bHandled = true;

	if (!m_data.size())
		return 0;

	RECT r;
	GetClientRect(&r);

	int left = r.left + 10;
	int right= r.right - 10;
	int bottom=r.bottom - bottom_margin;
	int top  = r.top + 10;

	int inc = (right - left) / m_data.size();

	dc.MoveTo(left,bottom);
	dc.LineTo(right,bottom);


	//+ve only for now, starting at 0...
	float max = *max_element(m_data.begin(),m_data.end());
	float min = *min_element(m_data.begin(),m_data.end());

	float scale;
	if (max == min)
		scale = 1;
	else 
		scale =  (top - bottom) / max;
	
	CBrushT<true> brushes[MAX_COLUMNS];

	for (i = 0;i<m_colors.size();i++)
		brushes[i].CreateSolidBrush(m_colors[i]);

	if (m_colors.size())
		::SelectObject(dc, brushes[0]);

	

	CFontT<true> font;
	int fsize = min(inc * 3,80);
	const TCHAR *szFace = m_FontFace.empty()?_T("MS Sans Serif (Western)"):m_FontFace.c_str();

	font.CreatePointFont(fsize,szFace,dc);
	::SelectObject(dc, font);

	double total = 0;
	for (i = 0;i<m_data.size();i++)
		total+=m_data[i];

	

	for (i = 0;i<m_data.size();i++)
	{
		
		if (m_colors.size())
		{
			int num = i%m_colors.size();
		
			::SelectObject(dc,brushes[num]);
		}

		RECT r;
		r.top = bottom + m_data[i] * scale;
		r.right= left + (i+1) * inc + 1;
		r.left = left + i * inc;
		r.bottom = bottom+1;

	
		dc.Rectangle(&r);

		if (total > 0)
		{
			int bkmode = dc.GetBkMode();
			dc.SetBkMode(TRANSPARENT);

			COLORREF old = dc.SetTextColor(RGB(255,255,255));
		
			double per = (m_data[i] * 100) / total;

			TCHAR out[255];
			if (m_acc > 0)
				_stprintf(out, _T("%.1f%"), per);
			else
				_stprintf(out, _T("%.0f%"), per);

		//	tstring out = _itot(per,b,10) + tstring(_T("%"));
			if ((r.bottom - r.top) > 10)
				dc.DrawText(out,-1,&r,DT_CENTER | DT_VCENTER | DT_SINGLELINE);
			dc.SetBkMode(bkmode);
			dc.SetTextColor(old);
		}

		if ( i < m_labels.size())
		{
			r.top = bottom+5;
			r.bottom = bottom + bottom_margin;
		
	
			dc.DrawText(m_labels[i].c_str(),-1,&r,DT_CENTER | DT_VCENTER);
		}
	}



		

	dc.RestoreDC(olddc);


	

	return 0;
}
