// ID3tag.h: interface for the CID3tag class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ID3TAG_H__8EE528D0_99D1_4FA3_A828_26C287B86D93__INCLUDED_)
#define AFX_ID3TAG_H__8EE528D0_99D1_4FA3_A828_26C287B86D93__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include <vector>
using namespace std;

#include <atlmisc.h>

#include "Callbacks.h"

class CID3Tag  
{

	static int Read7Int(const BYTE *pBuff);
	static int ReadInt(const BYTE *pBuff);

	static void WriteInt(BYTE *pBuff, int nVal);
	static void Write7Int(BYTE *pBuff, int nVal);

	struct CFrame
	{
		string buffer;
		char id[5];

		BYTE m_flags_high;
		BYTE m_flags_low;

		CFrame()
		{
			memset(id, 0, sizeof(id));
			m_flags_high = 0;
			m_flags_low  = 0;
		}


		int size() { return (10 + buffer.size()); }

		int Load(BYTE *pFrame)
		{

			memcpy(id,pFrame, 4);
			id[4] = 0;

			if (id[0]==0)
				return -1;

			pFrame+=4;
			int size = ReadInt(pFrame);

			//limit to 100KB to help spot errors...
			if (size > 100000)
				return -1;

			pFrame+=4;
			m_flags_high = *(pFrame + 0);
			m_flags_low  = *(pFrame + 1);
			buffer.assign((const char *)(pFrame)+2, size);
			
			return (size + 10);	
		}

		void Save(BYTE *pBuff)
		{
			memcpy(pBuff, id, 4);
			pBuff+=4;
			WriteInt(pBuff, buffer.size());
			pBuff+=4;
			*(pBuff + 0) = m_flags_high;
			*(pBuff + 1) = m_flags_low;
			pBuff+=2;
			memcpy(pBuff, buffer.data(), buffer.size());
		}

	};

	friend CFrame;
	


	CString m_sFname;
	BYTE m_nVersionMajor;
	BYTE m_nVersionRevision;
	BYTE m_nFlags;
	int	 m_nSize;

	BOOL m_bExtendedHeader;
	

	vector<CFrame> m_frames;

public:

	BOOL	m_bExists;

	BOOL	CreateNewSpace(int nSpace, iProgCallback *pCallback = NULL);
	BOOL	GetTextFrame(const string szID, string &data);
	int		GetFrameNumber(const string id);
	void	MakeTextFrame(const string szID, const string &data);
	BOOL	RemoveTagFromFile();
	void	MakeCueSheetFrame(const char *szCuesheet, const char *szVersion, const char *szInfo);
	int		GetCueFrameNumber(CString *pString = NULL);
	BOOL	GetCueFrame(CString &data);
	
	BOOL Load(const TCHAR *szFname);
	BOOL Save();
	CID3Tag();
	virtual ~CID3Tag();

private:
	
 
	
	

};

#endif // !defined(AFX_ID3TAG_H__8EE528D0_99D1_4FA3_A828_26C287B86D93__INCLUDED_)
