// InfoDlg.cpp: implementation of the InfoDlg class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"


#include <atlwin.h>
#include <atlframe.h>
#include <atlctrls.h>
#include <atldlgs.h>
#include <atlctrlw.h>

#include "InfoDlg.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CInfoDlg::CInfoDlg(tstring path, mp3data *pMp3, HWND hWndPar) : m_modehist(1), m_bithist(0), m_blockhist(1), m_modehist_present(TRUE), m_hWndPar(hWndPar), m_path(path), m_thread(path)
{

	if (pMp3)
	{
		m_fast = TRUE;
		m_mp3data = *pMp3;
	}
	else
		m_fast = FALSE;



}

CInfoDlg::~CInfoDlg()
{

}

LRESULT CInfoDlg::OnReport(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
{
	mp3data *pData = (mp3data *)lParam;
	m_mp3data = *pData;
	delete pData;

	ProcessMp3Data();

	return 0;
}

LRESULT CInfoDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{	


	HICON hIcon = LoadIcon(_Module.GetResourceInstance(),MAKEINTRESOURCE(IDR_MAINFRAME));
	SetIcon(hIcon);
	m_list = GetDlgItem(IDC_INFOLIST);
	m_list.SetExtendedListViewStyle(LVS_EX_FULLROWSELECT  ,LVS_EX_FULLROWSELECT );

	m_list.AddColumn(_T("Mp3 Attribute"),0);
	m_list.AddColumn(_T("Value"),1);
	
	CenterWindow(GetParent());
	SetWindowText(m_path.c_str());
	DlgResize_Init();

	CFontT<false> font = GetFont();
	LOGFONT lf;
	font.GetLogFont(&lf);

	tstring modelab[] = { _T("ss"),_T("is"),_T("ms"),_T("mixed") };

	m_modehist.m_colors.clear();
	m_modehist.m_colors.push_back(RGB(202,100,42));
	m_modehist.m_colors.push_back(RGB(227,130,61));
	m_modehist.SetLabels(modelab, 4);




	m_bithist.m_FontFace   = lf.lfFaceName;
	m_blockhist.m_FontFace = lf.lfFaceName;
	m_modehist.m_FontFace = lf.lfFaceName;

	m_blockhist.m_colors.clear();
	m_blockhist.m_colors.push_back(RGB(100,202,42));
	m_blockhist.m_colors.push_back(RGB(130,227,61));


	tstring block_labels[] = {_T("long"), _T("short"), _T("mixed")};
	m_blockhist.SetLabels(block_labels,3);


	m_bithist.Create(m_hWnd,rcDefault, NULL,  WS_CHILD );
	m_blockhist.Create(m_hWnd,rcDefault, NULL,  WS_CHILD );
	m_modehist.Create(m_hWnd,rcDefault, NULL,  WS_CHILD );
	MoveHistograms();

	m_bithist.ShowWindow(SW_SHOW);

	if (!m_fast)
	{
		m_thread.m_hWndPar = m_hWnd;
		m_thread.Begin();
	}
	else
	{
		SetEvent(m_thread.m_hEvent);
		ProcessMp3Data();
	}
	
	return TRUE;
}


void CInfoDlg::MoveHistograms()
{
	RECT rbit,rblock,rmode,r1,r2;
	GetClientRect(&r1);

	::GetWindowRect(GetDlgItem(IDC_GP1),&r2);
	ScreenToClient(&r2);

	int mid = 2 * (10 + r1.right) / 3;

	rbit.top = 20;
	rbit.bottom = r2.top - 10;
	rbit.left = 10;
	rbit.right = mid - 5;

	rblock = rbit;
	rblock.left = mid + 5;
	rblock.right= r1.right - 10;

	if (m_modehist_present)
	{
		rblock.bottom= rblock.top + (rbit.bottom - rbit.top) / 2 - 7;

		rmode = rblock;
		rmode.top = rblock.top + (rbit.bottom - rbit.top) / 2 + 9;
		rmode.bottom = rbit.bottom;

		
		m_blockhist.MoveWindow(&rblock);
		m_modehist.MoveWindow(&rmode);
	}
	else
	{
		rblock.bottom = rbit.bottom;
		m_blockhist.MoveWindow(&rblock);
	}

	m_bithist.MoveWindow(&rbit);

	rmode.bottom = rmode.top;
	rmode.top-=13;
	rmode.bottom-=1;
	CStatic joint = GetDlgItem(IDC_STATICJOINT);
	joint.MoveWindow(&rmode);

	rblock.bottom = rblock.top;
	rblock.top-=13;
	rblock.bottom-=1;
	CStatic block = GetDlgItem(IDC_STATICBLOCKS);
	block.MoveWindow(&rblock);

	
	

}

LRESULT CInfoDlg::OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{

	KillTimer(0);
	MoveHistograms();

	bHandled = false;
	SetTimer(0,500);
	ResizeColumns();
	return 0;
}

void CInfoDlg::ProcessMp3Data()
{

	mp3data_string data_string;
	data_string.update(m_mp3data, FALSE);

	if (  (data_string.mode!= _T("mono")) && (data_string.mode!= _T("stereo")))
	{
		m_modehist.ShowWindow(SW_SHOW);
		::ShowWindow(GetDlgItem(IDC_STATICJOINT),SW_SHOW);
		m_modehist_present = TRUE;
	
	}
	else
	{
		m_modehist.ShowWindow(SW_HIDE);
		::ShowWindow(GetDlgItem(IDC_STATICJOINT),SW_HIDE);
		m_modehist_present = FALSE;
		
	}

	MoveHistograms();




	

	m_blockhist.ShowWindow(SW_SHOW);
	TCHAR b[5];

	tstring encoder = data_string.encoder;
	
	
	//blockhist
	vector<float> blocks;
	for (int i = 0;i<3;i++)
		blocks.push_back(m_mp3data.blockCount[i]);

	m_blockhist.SetData(blocks);
	m_blockhist.InvalidateRect(NULL);







	//modehist
	vector<float> modes;
	for (i = 0;i<4;i++)
		modes.push_back(m_mp3data.modeCount[i]);

	m_modehist.SetData(modes);
	m_modehist.InvalidateRect(NULL);
	
	//bithist

	tstring lab[15];

	lab[0]=_T("Free");

	for (int k = 1;k<15;k++)
		lab[k] = _itot(tabsel_123[m_mp3data.base_lsf][m_mp3data.layer-1][k],b,10);

	//only Mpeg1 III supported at the moment
	m_bithist.SetLabels(lab,15);



	vector<float> bitrates;

	for (i = 0;i<16;i++)
		bitrates.push_back(m_mp3data.bitrateCount[i]);
	
	m_bithist.SetData(bitrates);
	m_bithist.InvalidateRect(NULL); 
	


	//checkboxes

	CheckDlgButton(IDC_CHECKPADDING,	m_mp3data.padding?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(IDC_CHECKSCALEFAC,	m_mp3data.scalefac?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(IDC_CHECKSCFSI,		m_mp3data.scfsi?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(IDC_CHECKCOPYRIGHT,	m_mp3data.copyright?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(IDC_CHECKORIGINAL,	m_mp3data.original?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(IDC_CHECKXING,		m_mp3data.xing_present?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(IDC_CHECKVBR,		m_mp3data.vbr?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(IDC_CHECKCOMPLETE,	m_mp3data.complete2?BST_CHECKED:BST_UNCHECKED);
	
	m_list.DeleteAllItems();
	
	AddToList(_T("Type"),data_string.type);
	AddToList(_T("Average Bitrate"),data_string.bitrate);
	AddToList(_T("Mode"),data_string.mode);
	AddToList(_T("Frequency"),data_string.freq+_T("Hz"));
	AddToList(_T("Number of Frames"),data_string.frames);
	AddToList(_T("Length"),data_string.length);
	AddToList(_T("Max Reservoir"),_itot(m_mp3data.max_reservoir,b,10));
	AddToList(_T("Sync Errors"),data_string.sync_errors);
	AddToList(_T("Emphasis"),data_string.emphasis);
	AddToList(_T("Scalefac"),data_string.scalefac);
	AddToList(_T("Encoder"),encoder);

	
	
	ResizeColumns();

}

void CInfoDlg::AddToList(tstring att, tstring val)
{
	int i = m_list.GetItemCount();
	m_list.AddItem(i,0,att.c_str());
	m_list.SetItemText(i,1,val.c_str());
}


LRESULT CInfoDlg::OnCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	mp3data_string data_string;
	data_string.update(m_mp3data, FALSE);

	TCHAR buff[256];
	TCHAR b[256];

	tstring tag = _T("--[ ") + tstring(g_version) + _T(" ]--[ http://www.guerillasoft.com ]--");
	int taglength = tag.size();

	tstring out;
	tstring name = m_path; 
	int pos = m_path.rfind(_T('\\'));
	if (pos!=string::npos)
		name = m_path.substr(pos+1,string::npos);
	
	out+=name + _T("\r\n");
	out+=tstring(name.size(),_T('-'))+_T("\r\n\r\n");
	
	
	
	out+= _T("Bitrates:\r\n");
	out+=tstring(taglength,_T('-'))+_T("\r\n");

	int max = 0;
	for (int i = 0;i<15;i++)
	{
		if (m_mp3data.bitrateCount[i]>max)
			max = m_mp3data.bitrateCount[i];
	}

	if (max == 0)
		max = 1;

	for (i = 0;i<15;i++)
	{
		int brate_value = tabsel_123[m_mp3data.base_lsf][m_mp3data.layer-1][i];
		int count		= m_mp3data.bitrateCount[i];

		if (count>0)
		{
	
		
			_stprintf(buff,_T("%i"), brate_value);
			tstring value = buff;
			value = tstring(max(3 - value.size(),0),_T(' ')) + value + _T("     ");	 
			out+=value;

			int num_bars = (40 * count)/ max;
			out+=tstring(num_bars,'|');
			out+=tstring(45 - num_bars,' ');

			_stprintf(buff,_T("%.1f%%"),float(count * 100) / m_mp3data.frameCount);
			tstring percent = buff;
			percent = tstring(max(7 - percent.size(),0),_T(' ')) + percent;
			out+=percent;
			out+=_T("\r\n");
		}

	}
	out+=tstring(taglength,_T('-'))+_T("\r\n");
	out+=_T("\r\n");

	pair<tstring,tstring> data_list[11]; 
	data_list[0] = make_pair<tstring,tstring>(_T("Type"),		data_string.type);
	data_list[1] = make_pair<tstring,tstring>(_T("Bitrate"),	_itot(m_mp3data.bitrate,b,10));
	data_list[2] = make_pair<tstring,tstring>(_T("Mode"),		data_string.mode);
	data_list[3] = make_pair<tstring,tstring>(_T("Frequency"),	_itot(m_mp3data.base_freq,b,10) + tstring(_T(" Hz")));
	data_list[4] = make_pair<tstring,tstring>(_T("Frames"),		_itot(m_mp3data.frameCount,b,10));
	data_list[5] = make_pair<tstring,tstring>(_T("Length"),		data_string.length);
	data_list[6] = make_pair<tstring,tstring>(_T("Max Reservoir"),_itot(m_mp3data.max_reservoir,b,10));
	data_list[7] = make_pair<tstring,tstring>(_T("Emphasis"),	data_string.emphasis);
	data_list[8] = make_pair<tstring,tstring>(_T("Scalefac"),	data_string.scalefac);
	data_list[9] = make_pair<tstring,tstring>(_T("Complete"),	m_mp3data.complete2?_T("yes"):_T("no"));
	data_list[10]= make_pair<tstring,tstring>(_T("Encoder"),	data_string.encoder);


	for (i = 0;i<11;i++)
	{
		tstring name = data_list[i].first;
		tstring data = data_list[i].second;

		out+=name+tstring(max(20-name.size(),0),_T(' ')) + _T(": ")+data + _T("\r\n"); 
	}

	out+=_T("\r\n");	
	out+=tag+_T("\r\n");
	if (OpenClipboard())
	{
		EmptyClipboard();

		
		HANDLE hGlobal = NULL;
		hGlobal = GlobalAlloc(GMEM_MOVEABLE, 
            (out.size() + 1) * sizeof(TCHAR)); 
		if (hGlobal)
		{
			LPTSTR lptstrCopy = reinterpret_cast<TCHAR *>(GlobalLock(hGlobal)); 
			memcpy(lptstrCopy, out.c_str(), 
				(out.size()+1) * sizeof(TCHAR)); 
	
			GlobalUnlock(hGlobal); 
 
		
			SetClipboardData(CF_TEXT, hGlobal);
		}

		CloseClipboard();
        
		
	}


	return 0;
}



LRESULT CInfoDlg::OnKeyDown(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (wParam==VK_ESCAPE)
	{
	
		KillTimer(0);
		DestroyWindow();
	}

	return 0;
}


