// IniFile.h: interface for the CIniFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_INIFILE_H__8D4657C1_04CA_11D5_A176_080046068885__INCLUDED_)
#define AFX_INIFILE_H__8D4657C1_04CA_11D5_A176_080046068885__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <iostream>
#include <fstream>

typedef basic_ifstream<TCHAR, char_traits<TCHAR> > tifstream;
typedef basic_ofstream<TCHAR, char_traits<TCHAR> > tofstream;

#define MAX_LINE 1024

 



class CIniFile  
{

	tstring m_fname;

	struct setting
	{
		tstring name;
		tstring value;
	};

	struct group
	{
		tstring name;
		vector<setting> settings;
	};


	vector<group> m_data;

	static void MakeMultibyte(char *szOut, const TCHAR *szIn)
	{

	#ifdef _UNICODE
				  WideCharToMultiByte(
				   CP_ACP,									// code page
				   0,										// dwFlags
				   szIn,								// lpWideCharStr
				   -1,										// cchWideCharStr
				   szOut,							// lpMultiByteStr
				   lstrlenW(szIn) + sizeof(WCHAR), // cchMultiByte
				   NULL,									// lpDefaultChar
				   NULL);									// lpUsedDefaultChar

	#else		  
				  lstrcpy(szOut, szIn);
	#endif


	}


public:
	CIniFile(tstring fname) : m_fname(fname)
	{
		
	}

	bool Read()
	{
		char fname[MAX_PATH];
		MakeMultibyte(fname, m_fname.c_str());
		tifstream file(fname);
		if (!file)
			return false;

		while (ReadGroup(file))
		{
			OutputDebugString("Read group...");
		}

		file.close();

		return true;
	}

	void Write()
	{
		char fname[MAX_PATH];
		MakeMultibyte(fname, m_fname.c_str());
		tofstream file(fname);
		if (!file)
			return;

		for (int i = 0;i<m_data.size();i++)
			WriteGroup(file, m_data[i]);


	}


	void SetString(tstring gpname, tstring name, tstring value)
	{
		for (int i = 0;i<m_data.size();i++)
			if (m_data[i].name == gpname) break;

		group *pgp = NULL;
		if (i == m_data.size())
		{
			group gp;
			gp.name = gpname;
			m_data.push_back(gp);
			pgp = &m_data[m_data.size()-1];
		}
		else
			pgp = &m_data[i];

		for (i = 0;i<pgp->settings.size();i++)
			if (pgp->settings[i].name == name) break;

	
		setting set;
		set.name = name;
		set.value= value;

		if (i == pgp->settings.size())
			pgp->settings.push_back(set);
		else
			pgp->settings[i] = set;
	}

	tstring GetString(tstring gpname, tstring name, tstring def) const
	{
		tstring setting;
		return GetSetting(gpname, name, setting)?setting:def;
	}


	void SetInt(tstring gpname, tstring name, int value)
	{
		TCHAR b[100];
		SetString(gpname, name, _itot(value, b, 10));
	}

	int GetInt(tstring gpname, tstring name,int def) const
	{
		tstring setting;
		return GetSetting(gpname, name, setting)?_ttoi(setting.c_str()):def;
	}

	
	virtual ~CIniFile() {}

private:
	
	BOOL GetSetting(tstring gpname, tstring name, tstring &setting) const
	{
		for (int i = 0;i<m_data.size();i++)
			if (m_data[i].name == gpname) break;

		if (i == m_data.size()) return FALSE;

		const group &gp = m_data[i];
		for (i = 0;i<gp.settings.size();i++)
			if (gp.settings[i].name == name) break;

		if (i == gp.settings.size() ) 
			return FALSE;
		else 
			setting = gp.settings[i].value;

		return TRUE;
	}



	

	setting MakeSetting(const tstring &in)
	{
		setting ret;
		int pos = in.find(_T("\t"));
		if (pos == tstring::npos)
		{
			ret.name = _T("corrupt");
			return ret;
		}


		ret.name.assign(in,0,pos);
		ret.value.assign(in,pos+1,tstring::npos);

		return ret;
	}

	void WriteGroup(tofstream &file, const group &gp)
	{
		file<<gp.name<<_T("\n");
		for (int i=0;i<gp.settings.size();i++)
			file<<gp.settings[i].name<<_T("\t")<<gp.settings[i].value<<_T("\n");

		file<<_T("\n");
	}


	bool ReadGroup(tifstream &file)
	{
		group gp;
		TCHAR line[MAX_LINE];

		file.getline(line,sizeof(line));
		if ( (file.eof()) || lstrlen(line) == 0 )
			return false;

		gp.name = line;

		for (;;)
		{
			file.getline(line,sizeof(line));
			if (file.eof())
				return false;
			tstring l = line;

			if (l.empty())
			{
				m_data.push_back(gp);
				return true;
			}
			else
				gp.settings.push_back(MakeSetting(l));
		}

		


	}

};

#endif // !defined(AFX_INIFILE_H__8D4657C1_04CA_11D5_A176_080046068885__INCLUDED_)
