// ListThread.cpp: implementation of the CListThread class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ListThread.h"

#include "encspot.h"
#include "mp3file.h"


BOOL CListThread::DoProcess(string path, mp3data &data, mp3data_string &data_string, LPLVITEMDATA lplvid, int cap)
{
	LOG(_T("CListThread::DoProcess"));

	//callback sink for CMp3File::ProcessFrames	
	struct thread_helper : public CMp3File::sink
	{
		HANDLE hAbort;
		BOOL Fire(const mp3data &data) const
		{
			if (WaitForSingleObject(hAbort,0) == WAIT_OBJECT_0)
			{
			
				return FALSE;
			}
			else
				return TRUE;
		}
	};

	if (cap == 0)
		cap = -1;	//read all frames

	try
	{
		data.all_read = FALSE;

		thread_helper helper;
		helper.hAbort = m_hEventAbort;

		CMp3File mp3;
		if (!mp3.Open(path))
		{
			::PostMessage(m_hWndMain, WM_BADFILE, (WPARAM)lplvid,NULL);
			return FALSE;
		}
		
		if (!mp3.ProcessFrames(data,cap, &helper))
			return TRUE;

		data_string.update(data, *m_pMonoQuality);

		//unknown with fast scan: do more scanning
		 if (!data.all_read && *m_pReadAllUnknown && data_string.encoder == _T("Unknown"))
		 {
			 if (!mp3.ProcessFrames(data, -1, &helper))
				 return TRUE;
		 }
	}
	catch(TCHAR *msg)
	{
		LOG(_T("CListThread::SetupItem exception in CMp3File::ProcessFrames ")+tstring(msg));
		throw;
	}

	data_string.update(data, *m_pMonoQuality);
	return TRUE;

}

BOOL CListThread::OnProcessSingle(WPARAM wParam, LPARAM lParam)
{
	LOG(_T("CListThread::OnProcessSingle"));

	int frame_res = LOWORD(wParam);
	DWORD dwCookie = HIWORD(wParam);

	BOOL success = OnProcess(frame_res, lParam);
	
	if (WaitForSingleObject(m_hEventAbort,0) == WAIT_OBJECT_0)
	{
		ResetEvent(m_hEventAbort);
		return TRUE;
	}
	else if (success)
	{
		LOG(_T("CListThread::OnProcessMulti posting message"));
		::PostMessage(m_hWndMain, WM_THREADDONE,1,(LPARAM)lParam);
	}

	return TRUE;	
}

BOOL CListThread::OnProcessMulti(WPARAM wParam, LPARAM lParam)
{
	LOG(_T("CListThread::OnProcessMulti"));

	vector<LPLVITEMDATA> *pList = reinterpret_cast<vector<LPLVITEMDATA> *>(lParam);
	vector<LPLVITEMDATA> datalist = *pList;
	delete pList;

	

	int frame_res = LOWORD(wParam);
	DWORD dwCookie = HIWORD(wParam);

	int size = datalist.size();

	for (int i = 0;i<size;i++)
	{
		LPARAM lpData = reinterpret_cast<LPARAM>(datalist[i]);
		BOOL success = OnProcess(frame_res, lpData);
	
		if (WaitForSingleObject(m_hEventAbort,0) == WAIT_OBJECT_0)
		{
			ResetEvent(m_hEventAbort);
			return TRUE;
		}
		else if (success)
		{
			LOG(_T("CListThread::OnProcessMulti posting message "));
			::PostMessage(m_hWndMain, WM_THREADDONE,MAKELONG((i==(datalist.size()-1)), dwCookie),lpData);
		}
	}

	return TRUE;

}

BOOL CListThread::OnProcess(WPARAM wParam, LPARAM lParam) 
{
	LOG(_T("CListThread::OnProcess"));

	int frame_resolution = wParam;

	
	LPLVITEMDATA lplvid = (LPLVITEMDATA)lParam;

	tstring path = lplvid->path;

	LOG(_T("CListThread::OnProcess ")+tstring(path));

	mp3data &data = lplvid->data;
	mp3data_string &data_string = lplvid->data_string;
	try
	{
		return DoProcess(path, data, data_string, lplvid, frame_resolution);	
	}
	catch(...)
	{
		LOG(_T("CListThread::OnProcess unspecified exception "));			
	}

	data.length = 0;

	data_string.encoder = _T("--");
	data_string.bitrate = _T("--");
	data_string.fsize = _T("--");	
	data_string.quality = 0;
	data_string.mode = _T("--");

	return TRUE;

}


void CListThread::StopEverythingNow()
{
	if (IsIdle())
		return;

	Abort();
	EmptyQueue();

	if (!WaitForIdle(1000))
		MessageBox(GetActiveWindow(), _T("Thread is a bit _too_ busy..."
										 "Please report this..."),0,0);
}


