// Mp3File.cpp: implementation of the CMp3File class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Mp3File.h"

#include "encspot.h"

const long freqs[9] = { 44100, 48000, 32000,
			 22050, 24000, 16000 ,
			 11025 , 12000 , 8000 };
  
 const int tabsel_123[2][3][16] = {
  { {128,32,64,96,128,160,192,224,256,288,320,352,384,416,448,},
    {128,32,48,56, 64, 80, 96,112,128,160,192,224,256,320,384,},
    {128,32,40,48, 56, 64, 80, 96,112,128,160,192,224,256,320,} },

  { {128,32,48,56,64,80,96,112,128,144,160,176,192,224,256,},
    {128,8,16,24,32,40,48,56,64,80,96,112,128,144,160,},
    {128,8,16,24,32,40,48,56,64,80,96,112,128,144,160,} }
};

int reverse_int(BYTE b[]);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BOOL CMp3File::Open(tstring fname)
{
	m_tsFname = fname;

	m_tag.Load(fname.c_str());

	//open for reading only
	m_pFile = fopen(fname.c_str(),_T("rb"));

	if (m_pFile)
	{
		fseek(m_pFile,0,SEEK_END);
		m_nFilesize = ftell(m_pFile);
		fseek(m_pFile,0,SEEK_SET);
	}
	
	return (m_pFile!=NULL);
}


void CMp3File::Close()
{
	fclose(m_pFile);
}


CMp3File::CMp3File() : m_pFile(NULL)
{

}

CMp3File::~CMp3File()
{
	if (m_pFile)
		fclose(m_pFile);
	

}

int CMp3File::ReadInt()
{

	int head = 0;
	head |= getc(m_pFile); head = head << 8;
	head |= getc(m_pFile); head = head << 8;
	head |= getc(m_pFile); head = head << 8;
	head |= getc(m_pFile);

	return head;
}


int CMp3File::ReadInt(BYTE *pBuffer)
{
	int head = 0;
	head |= *(pBuffer + 0); head = head << 8;
	head |= *(pBuffer + 1); head = head << 8;
	head |= *(pBuffer + 2); head = head << 8;
	head |= *(pBuffer + 3);

	return head;
}



int CMp3File::MakeHeader(unsigned long head, mp3header &mhHead)
{
	int srate;
	int framesize;

	if ((head & 0xffe00000) != 0xffe00000) return 0;
	if (!((head>>17)&3)) return 0;
	if (((head>>12)&0xf) == 0xf) return 0;
	if (((head>>10)&0x3) == 0x3 ) return 0;

	//mp3 type id
  
	mhHead.id			= (head >> 19) &  1;
	mhHead.idex			= (head >> 20) &  1;		//0 = Mpeg 2.5

	if(!mhHead.idex) 
		srate			= 6 + ((head>>10)&0x3);		
	else 
		srate			= ((head>>10)&0x3) + ((1-mhHead.id) * 3);

	mhHead.bitrate_idx	= ((head>>12)&0xf);
	mhHead.padding		= ((head>>9)&0x1);
	mhHead.mode			= ((head>>6)&0x3);
	mhHead.modeex		= (head >>  4) &  3; 

	mhHead.freq			= freqs[srate];
	mhHead.channels		= (mhHead.mode == MPG_MD_MONO) ? 1 : 2;
	mhHead.layer		= 4-((head>>17)&3);
	mhHead.protection	= (head >> 16) &  1;
	mhHead.copyright	= (head >> 3) & 1;
	mhHead.original		= (head >> 2) & 1;
	mhHead.emphasis		= head & 3;

	framesize = tabsel_123[1 - mhHead.id][mhHead.layer-1][mhHead.bitrate_idx]*144000/(mhHead.freq<<(1 - mhHead.id))+mhHead.padding;
	return framesize;
}





int	CMp3File::SeekNextHeader(int safety_max)
{
	MYASSERT(m_pFile);

	int pos = ftell(m_pFile);
	mp3header mhHead;

	BYTE buffer[65536 * 2];
	int read = fread(buffer,1,sizeof(buffer),m_pFile);


	for(int i=0;i<read/2;i++)
	{
		int framesize_guess = 1;
		int safety = 0;
		int j = i;

		for (safety = 0;(safety < safety_max) && framesize_guess;safety++)
		{
			int head		= ReadInt(buffer + j);			
			framesize_guess = MakeHeader(head, mhHead);

			j+=framesize_guess;

			if ((j + 4) >= read)
				return -1;
		}

		if (safety==safety_max)
		{
			fseek(m_pFile, pos + i, SEEK_SET);
			return i;
		}
	}

	fseek(m_pFile, pos, SEEK_SET);
	return -1;
}




int	CMp3File::SeekLastHeader()
{
	MYASSERT(m_pFile);

	int pos = ftell(m_pFile);
	
	fseek(m_pFile, 0, SEEK_END);
	int fsize = ftell(m_pFile);

	//look at up to last 2MB
	for (int i = 0;i<5;i++)
	{
		int back_pos = min(fsize, 65536 * 5);
		fseek(m_pFile, -back_pos, SEEK_END);

		if (SeekNextHeader()!=-1)
			break;
		
	}

	if (i==5)
		return -1;

	

	for (;;)
	{
		int currpos = ftell(m_pFile);
	
		int head = ReadInt();
		mp3header mhHead;

		//if we had a a frame, but now dont, then perhaps id3 tag....
		int framesize = MakeHeader(head, mhHead);
		if (!framesize)
			return 1;

		

		if ((currpos + framesize) == fsize)
			return 1;
		else if ((currpos + framesize) > fsize)
			return -1;
		else
			fseek(m_pFile,currpos + framesize,SEEK_SET);
	}



}

BOOL CMp3File::ProcessFrames(mp3data &data_out, int nFrameLimit, const CMp3File::sink *pHelp)
{
	MYASSERT(m_pFile);
	fseek(m_pFile, 0, SEEK_SET);

	tstring label = GetLabels();
	if (!label.empty())
		lstrcpy(data_out.label,label.c_str());
	else
		data_out.label[0] = 0;


	data_out.complete_old	= true;	//false if part of frame found...

	data_out.complete2   = (SeekLastHeader()!=-1);
	fseek(m_pFile, 0, SEEK_SET);

	int framelength_debug = GuessFrameLength();
	fseek(m_pFile, 0, SEEK_SET);


	data_out.all_read	= FALSE;
	data_out.fsize		= m_nFilesize;

	m_lastBits = 0;

	if (m_nFilesize==0)
		return FALSE;

	int		id3v1pos	= CheckID3v1();
	BOOL	bHasID3v2	= m_tag.m_bExists;
	int num = m_tag.GetCueFrameNumber();
	data_out.cue		= (num!=-1);

	data_out.xing_present = GetXing(data_out.xing_header);


	//locate first header
	if (SeekNextHeader()==-1)
		throw "Cannot find valid mp3 header, scanning failed\n";
	int pos = ftell(m_pFile);

	//read first header
	int nFirstHeader		= ReadInt();
	mp3header hFirstHeader;
	int nFirstFramesize		= MakeHeader(nFirstHeader,hFirstHeader);
	

	if (!nFirstFramesize)
		throw	"Hmm - I thought I'd found a header... \r\n"
				"Please report this bug...\r\n";

	SetBaseInfo(data_out, hFirstHeader);

	//prepare for the main loop

	memset(data_out.bitrateCount,0,sizeof(data_out.bitrateCount));
	memset(data_out.modeCount,0,sizeof(data_out.modeCount));
	memset(data_out.blockCount,0,sizeof(data_out.blockCount));

	data_out.frameCount = 0;

	BOOL	bResync		= FALSE;
	

	pos+=nFirstFramesize;
	for (;;)
	{
		
		if (bResync) 
		{
			LOG(_T("Sync Error: ") + m_tsFname);
			data_out.sync_errors++;
			if (SeekNextHeader()==-1)
				break;
			bResync = FALSE;
		}
		else
			fseek(m_pFile,pos, SEEK_SET);

		if (pos==id3v1pos)
			break;

		if (CheckDone(data_out, pos))
			break;

		int newpos = ProcessNextFrame(data_out);
		if (newpos==-1)
			bResync = TRUE;
		else
			pos = newpos;

		++data_out.frameCount;

		if (pHelp && (data_out.frameCount % 2000) == 0)
		{
			if (!ReportProgress(data_out, pHelp))
				return FALSE;
		}
		
		if (nFrameLimit!=-1 && data_out.frameCount >= nFrameLimit)
			break;
	}

	if (nFrameLimit == -1 || data_out.frameCount < nFrameLimit)
		data_out.all_read = 1;

	if (!ReportProgress(data_out, pHelp))
		return FALSE;

	return TRUE;
}


BOOL CMp3File::ReportProgress(mp3data &data, const CMp3File::sink *pHelp)
{

	if (data.all_read)
		data.length			= int(data.frameCount*576.0*(data.base_lsf?1:2)/data.base_freq); 			
	else	//estimate
	{
		float average_bitrate = GetAverageBitrate(data);
		data.length = (data.fsize * 8)/(1000 * average_bitrate);
	}

	data.bitrate = GetAverageBitrate(data);

		

	//is the file a vbr file?
	DetectVBR(data);

	if (pHelp)
		return pHelp->Fire(data);

	return TRUE;

}

int CMp3File::ProcessNextFrame(mp3data &data_out)
{
	int  nStartPos = ftell(m_pFile);

	//read next header
	int nHeader		= ReadInt();
	mp3header hHeader;
	int nFramesize		= MakeHeader(nHeader,hHeader);

	if (!nFramesize)
		return -1;

	tstring err = ReportOutOfSyncError(data_out, hHeader);
	if (!err.empty())
	{
//		printf("%s  at %d(%d%%), frame number %d\n",err.c_str(), pos,pos*100/out_data.fsize,framenum);
		return -1;
	}

	UpdateInfoFromHeader(data_out, hHeader);

	for(int gr=0;gr<((1 - hHeader.id)?1:2);gr++)
	{
		
		for(int ch=0;ch<hHeader.channels;ch++)
		{
		  int region,window,window_switching_flag;
		  readBits(12);					// part2_3_length
		  int big_values = readBits(9); // big_values
		  readBits(8);					// global_gain
		  if (hHeader.id == 1) 
			  readBits(4);				// scalefac_compress
		  else 
			  readBits(9);				// scalefac_compress
		  window_switching_flag = readBits(1);
		  if (window_switching_flag == 1) 
		  {
			int block_type = readBits(2);					// block_type
			int mixed_block_flag = readBits(1);				// mixed_block_flag
			for(region=0;region<2;region++) readBits(5);	// table_select
			for(window=0;window<3;window++) readBits(3);	// subblock_gain
			if (block_type == 2) 
			{
				  if (mixed_block_flag) 				  
					data_out.blockCount[2]++;
				  else 
					data_out.blockCount[1]++;
			} 
			else 
				data_out.blockCount[0]++;
		  } //window switching
		  else 
		  {
			int region;
			for(region=0;region<3;region++) readBits(5); // table_select
			readBits(4); // region0_count
			readBits(3); // region1_count
			data_out.blockCount[0]++;
		  }
		  if (hHeader.id == 1) readBits(1); // preflag
		  if (readBits(1))
			data_out.scalefac++; 
		  readBits(1); // count1table_select
		}//channel
	}//granules

	data_out.bitrateCount[hHeader.bitrate_idx]++;
	if (hHeader.mode == 1) 
		data_out.modeCount[hHeader.modeex]++;
	else 
		data_out.modeCount[0]++;

//		other_data.crcCount+=hHeader.protection;
	
	
	int framesize = tabsel_123[1 - hHeader.id][hHeader.layer-1][hHeader.bitrate_idx]*144000/(hHeader.freq<<(1 - hHeader.id)) + hHeader.padding;
	
	
	return (nStartPos + framesize);
}



void CMp3File::UpdateInfoFromHeader(mp3data &out_data, const mp3header &hHeader)
{
	m_nLastBits = 0;
	if (!hHeader.protection) readBits(16);		//!protection =>16 bit CRC follows header
	if (hHeader.padding) out_data.padding = 1;

	if (hHeader.id) 
	{
		int main_data_begin = readBits(9);
		if (main_data_begin > out_data.max_reservoir) 
			out_data.max_reservoir = main_data_begin;
	} 
	else 
	{
		int main_data_begin = readBits(8);
		if (main_data_begin > out_data.max_reservoir) 
			out_data.max_reservoir = main_data_begin;
	}

	if (hHeader.id) 
	{

		int test;
		int pos = ftell(m_pFile);
		if (hHeader.mode==3) 
			test = readBits(5); // private_bits
		else 
			test = readBits(3); // private_bits
	} 
	else 
	{
		if (hHeader.mode==3) readBits(1); // private_bits
		else readBits(2); // private_bits
	}

	if (hHeader.id == 1) 
	{
		int ch,scfsi_band;
		for(ch=0;ch<hHeader.channels;ch++)
		  for(scfsi_band=0;scfsi_band<4;scfsi_band++)
			out_data.scfsi = readBits(1) || out_data.scfsi;
	}

}





void CMp3File::SetBaseInfo(mp3data &data, const mp3header &header)
{
	data.mode		= header.mode;
	data.nch		= header.channels;
	data.base_lsf	= 1 - header.id;  //change this...
	data.layer		= header.layer;
	data.base_freq  = header.freq;

	if (header.bitrate_idx == 0)
		throw	"This file seems free format bitstream.\n"
				"Free format bitstream is not supported.\n"
				"Sorry.\n";

	data.copyright	= header.copyright;
	data.original	= header.original;
	data.emphasis	= header.emphasis;
 
}


int CMp3File::CheckID3v1()
{

	int id3v1pos = -1;

	unsigned char c1,c2,c3;
	fseek(m_pFile,-128,SEEK_END);
	c1 = getc(m_pFile);
	c2 = getc(m_pFile);
	c3 = getc(m_pFile);
	if (c1 == 'T' && c2 == 'A' && c3 == 'G') 
		id3v1pos = m_nFilesize - 128;
  

	fseek(m_pFile,0,SEEK_SET);

	return id3v1pos;
}

BOOL CMp3File::GetID3v2()
{

	fseek(m_pFile,0,SEEK_SET);
	unsigned char c1,c2,c3,c4;

	c1 = getc(m_pFile);
	c2 = getc(m_pFile);
	c3 = getc(m_pFile);


	if (c1 == 'I' && c2 == 'D' && c3 == '3') 
	{
	  fseek(m_pFile,6,SEEK_SET);
	  c1 = getc(m_pFile);
	  c2 = getc(m_pFile);
	  c3 = getc(m_pFile);
	  c4 = getc(m_pFile);
	  fseek(m_pFile,c1*2097152+c2*16384+c3*128+c4,SEEK_CUR);
	  return TRUE;
	}
	else
	{
		fseek(m_pFile,0,SEEK_SET);
		return FALSE;
	}
}

BOOL CMp3File::GetXing(xing &out_data)
{

	int posn = ftell(m_pFile);

	BYTE buff[2000];
	fread(buff,1,sizeof(buff),m_pFile);
	fseek(m_pFile, posn, SEEK_SET);

	int pos = 0;
	while (  (pos <= (sizeof(buff) - 4)) && memcmp(buff + pos, "Xing", 4) )
		pos++;

	if (pos > sizeof(buff) - sizeof(out_data)) return FALSE;

	memcpy(&out_data,buff+pos,sizeof(out_data));

	//correct order...=0
	out_data.flags = reverse_int(buff+pos+4);
	out_data.frames= reverse_int(buff+pos+8);

	return TRUE;
}

BOOL CMp3File::CheckDone(mp3data &data, int pos)
{
	if (pos > data.fsize)		//pos is supposed to be the end of the last frame
	{								//so the last frame was truncated...
		data.complete_old = false;
		return TRUE;
	}
	else if (pos  == data.fsize)
		return TRUE;				//all done
	else
		return FALSE;

}

tstring CMp3File::ReportOutOfSyncError(mp3data &main_data, const mp3header &hHeader)
{

	tstring ret;

	if (hHeader.layer != main_data.layer) 
	{
		ret = _T("Layer error");
		return ret;
	}

	if (hHeader.channels != main_data.nch) 
	{
		ret=_T("Channel mode error"); 
		main_data.nch = hHeader.channels;
		return ret;
	}

	if (hHeader.freq != main_data.base_freq) 
	{
		ret=_T("Frequency error");
		main_data.base_freq = hHeader.freq;
		return ret;
	}

	if (hHeader.bitrate_idx == 0) 
	{
		ret = _T("Freeformat frame");
		return ret;
	}

	return ret;

}

BOOL CMp3File::DetectVBR(mp3data &data_out)
{
	bool vbr(false);
	int v1(0), v2(0);
	int i=1;
	while ((v1==0) && (i<16)){
	  if (data_out.bitrateCount[i] != 0)
		  v1=i;
	  i++;
	}
	while ((v2==0) && (i<16)){
	  if (data_out.bitrateCount[i] != 0)
		  v2=i;
	  i++;
	}
	if (v2!=0)
	  vbr=true;
	data_out.vbr=vbr;

	return vbr;
}

tstring CMp3File::GetLabels()
{
	tstring ret;
	char buff[1024]; 
	
	fseek(m_pFile, 0, SEEK_SET);
	GetID3v2();

	int count = fread(buff,1,sizeof(buff),m_pFile);
	ret = GetLabelsFromBuffer(buff,sizeof(buff));
	if (!ret.empty())
		return ret;


	fseek(m_pFile,-1024,SEEK_END);
	fread(buff,1,sizeof(buff),m_pFile);
	fseek(m_pFile, 0, SEEK_SET);

	
	return GetLabelsFromBuffer(buff,sizeof(buff));
}


tstring CMp3File::GetLabelsFromBuffer(const char *buff, int len)
{
	USES_CONVERSION;

	string b(buff,len);
	int pos = b.find("LAME");

	CHAR version[256];
	lstrcpy(version, b.substr(pos+4,4).c_str());

	if ( (pos!=string::npos) && (pos < (sizeof(buff) - 8)))
		return(_T("Lame ")+ tstring(A2T(version)));
	else if (b.find("VBRI")!=string::npos)
		return _T("FhG");
	else return _T("");
}




unsigned long int CMp3File::readBits(int n)
{
  unsigned long int ret = 0;
  int k;
  unsigned char c;

  while(m_nLastBits <= 24) {
    int tmp;
    tmp = getc(m_pFile);
    c = tmp;
    m_lastBits = m_lastBits << 8;
    if (tmp != EOF) m_lastBits = m_lastBits | c;
    m_nLastBits += 8;
  }

  if (n <= m_nLastBits) k = n; else k = m_nLastBits;

  ret = (m_lastBits >> (m_nLastBits - k)) & ((1 << k)-1);
  m_nLastBits -= k;
  n -= k;

  while(m_nLastBits <= 24) {
    int tmp;
    tmp = getc(m_pFile);
    c = tmp;
    m_lastBits = m_lastBits << 8;
    if (tmp != EOF) m_lastBits = m_lastBits | c;
    m_nLastBits += 8;
  }

  ret = ret << n;
  ret = ret | ((m_lastBits >> (m_nLastBits - n)) & ((1 << n)-1));
  m_nLastBits -= n;
	
  return ret;
}

float CMp3File::GetAverageBitrate(const mp3data &data)
{
	if (data.xing_present)
	{
		int frames		= data.xing_header.frames;
		int samples;
		if (data.layer==1)
			samples = frames * 384;
		else
			samples = frames * 1152;

		float length      = samples/data.base_freq;
		return (data.fsize * 8/(1000 * length)); 
		
		
	}

	float average = 0;
	int total = 0;
	for (int i = 1;i<16;i++)
	{
		average+=data.bitrateCount[i] * tabsel_123[data.base_lsf][data.layer-1][i];
		total  +=data.bitrateCount[i];
	}

	if (total == 0)
		return 0;
	average/=total;
	return average;

}

int CMp3File::GuessFrameLength()
{
	SeekNextHeader();
	int pos0 = ftell(m_pFile);
	fseek(m_pFile, pos0+1, SEEK_SET);
	SeekNextHeader();
	int pos1 = ftell(m_pFile);

	return (pos1 - pos0);

}




int reverse_int(BYTE b[])
{

	return b[3] + b[2] * 0x100 + b[1] * 0x10000 + b[0] * 0x1000000;
}


