#pragma once

#include "id3tag.h"

#define MPG_MD_STEREO       0
#define MPG_MD_JOINT_STEREO 1
#define MPG_MD_DUAL_CHANNEL 2
#define MPG_MD_MONO         3




#define MPEGID_1			0x0		//MPEG Version 1 (ISO/IEC 11172-3) 	
#define MPEGID_2			0x1		//MPEG Version 2 (ISO/IEC 13818-3)

struct mp3header
{
	int id;							//e.g. MPEG version 1, unless idex = 1
	int idex;						//1 = MPEG 2.5
	int bitrate_idx;				//between 0 and 15
	int mode;						//e.g. joint stereo
	int modeex;						//e.g. joint stereo usage
	int padding;					//0 or 1
	int freq;						//e.g. 44100
	int channels;					//e.g. 1 = mono, 2 = stereo
	int layer;						//1,2 or 3	
	int protection;					//crc protection bit

	int copyright;
	int emphasis;
	int original;
};
 
struct mp3data;
struct xing;
struct thread_helper;

class CMp3File  
{
	FILE *		m_pFile;
	tstring		m_tsFname; 
	int			m_nFilesize;
	int			m_nLastBits;
	int			m_lastBits;

public:

	

	struct sink
	{
		virtual BOOL Fire(const mp3data &data) const = 0;
	};

	CID3Tag	m_tag;

	float GetAverageBitrate(const mp3data &data);
	BOOL ReportProgress(mp3data &data, const sink *pHelp);
	BOOL DetectVBR(mp3data &data_out);
	void UpdateInfoFromHeader(mp3data &out_data, const mp3header &hHeader);
	BOOL ProcessFrames(mp3data &data_out, int nFrameLimit = -1, const sink *pHelp = NULL);
	void Close();
	CMp3File();
	virtual ~CMp3File();

	BOOL	Open(tstring fname);
	int		SeekNextHeader(int safety_max = 5);
	int		SeekLastHeader();

private:
	int GuessFrameLength();
	BOOL	ProcessNextFrame(mp3data &data_out);
	tstring GetLabelsFromBuffer(const char *buff, int len);
	tstring ReportOutOfSyncError(mp3data &main_data, const mp3header &hHeader);
	BOOL CheckDone(mp3data &data, int pos);
	void SetBaseInfo(mp3data &data, const mp3header &header);

	
	tstring GetLabels();

	int ReadInt();
	int ReadInt(BYTE *pBuffer);

	//used to be head_check2
	int MakeHeader(unsigned long head, mp3header &mhHead);

	BOOL GetXing(xing &out_data);
	BOOL GetID3v2();
	int CheckID3v1();

	unsigned long int readBits(int n);


};

