#include "stdafx.h"
#include <algorithm>

#include <atlwin.h>
#include <atldlgs.h>
#include <atlframe.h>
#include <atlddx.h>		
#include <atlctrls.h>
#include <atlctrlw.h>

#include "mainfrm.h"
#include "aboutdlg.h"

#include "mp3file.h"

#include "EncSpotLib/WinFactory.h"

CMainFrame::CMainFrame(CWindow *pParent) 
{ 
	m_pView = CWinFactory::Instance().Create(WID_VIEW, this);

	TCHAR fname[MAX_PATH];
	::GetModuleFileName(_Module.m_hInst,fname, MAX_PATH);
	PathRemoveExtension(fname);
	CIniFile ini(tstring(fname) + _T(".ini"));
	ini.Read();

	LoadSettings(ini); 
}
CMainFrame::~CMainFrame() 
{ 

	TCHAR fname[MAX_PATH];
	::GetModuleFileName(_Module.m_hInst,fname, MAX_PATH);
	PathRemoveExtension(fname);
	CIniFile ini(tstring(fname) + _T(".ini"));

	SaveSettings(ini);
	
	ini.Write();
	delete m_pView;
}



LRESULT CMainFrame::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{

	bHandled = FALSE;
	return 0;
}



LRESULT CMainFrame::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	LOG(g_version);
	BOOL max_store = m_bMaximize;
	RECT r_store = m_InitRect;
	
	// create command bar window
	HWND hWndCmdBar = m_CmdBar.Create(m_hWnd, rcDefault, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE);
	// atach menu
	m_CmdBar.AttachMenu(GetMenu());
	// load command bar images
//	m_CmdBar.LoadImages(IDB_TOOL);
	// remove old menu
	SetMenu(NULL);


	CToolBarCtrl toolbar = CreateSimpleToolBarCtrl(m_hWnd, IDR_MAINFRAME, FALSE, ATL_SIMPLE_TOOLBAR_PANE_STYLE);
	toolbar.SetButtonSize(18,23);
	
	CreateSimpleReBar(ATL_SIMPLE_REBAR_STYLE);
	AddSimpleReBarBand(hWndCmdBar);
	AddSimpleReBarBand(toolbar, NULL, TRUE);
	CreateSimpleStatusBar();

	m_status = m_hWndStatusBar;
	m_status.SetSimple(FALSE);
	int parts[] = {200,-1,};
	m_status.SetParts(2,parts);

	RECT r;
	GetClientRect(&r);

	CFrameWindowStuff *pFW = dynamic_cast<CFrameWindowStuff *>(m_pView);
	m_hWndClient = pFW->CreateImp(m_hWnd, &r);

	UIAddToolBar(toolbar);
	UISetCheck(ID_VIEW_TOOLBAR, 1);
	UISetCheck(ID_VIEW_STATUS_BAR, 1);
	UISetCheck(ID_LOGGING, g_logging);

	CMessageLoop* pLoop = _Module.GetMessageLoop();
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

	



	m_toolimages = ImageList_Create(18,23,ILC_COLOR8|ILC_MASK,9,1);
	HBITMAP hBitmap =  (HBITMAP)LoadImage(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDB_TOOL),IMAGE_BITMAP,0,0,LR_CREATEDIBSECTION);
	ImageList_AddMasked(m_toolimages,hBitmap,RGB(255,0,255));
	m_toolimages.SetBkColor(CLR_NONE);
	DeleteObject(hBitmap);	
	toolbar.SetImageList(m_toolimages);

	m_animate.Create(m_hWnd,NULL,NULL, WS_BORDER | WS_CHILD, 0);
	BOOL s = m_animate.Open(MAKEINTRESOURCE(IDR_ANIM));

	BOOL success = AddComplicatedReBarBandCtrl(m_hWndToolBar, m_animate, 0, NULL, 
		FALSE,40, TRUE, RBBS_FIXEDSIZE | RBBS_NOGRIPPER);
	

	MoveWindow(&r_store);
	if (max_store)
		ShowWindow(SW_MAXIMIZE);
	else
		ShowWindow(SW_SHOW); 

	return 0;
}


//same as in WTL AddSimpleReBarBandCtrl, but allows different styles of band
BOOL CMainFrame::AddComplicatedReBarBandCtrl(HWND hWndReBar, 
											 HWND hWndBand, 
											 int nID, 
											 LPTSTR lpstrTitle, 
											 BOOL bNewRow, 
											 int cxWidth, 
											 BOOL bFullWidthAlways, 
											 UINT fExtraStyles )
{

	ATLASSERT(::IsWindow(hWndReBar));	// must be already created

		ATLASSERT(::IsWindow(hWndBand));	// must be already created

		// Get number of buttons on the toolbar
		int nBtnCount = (int)::SendMessage(hWndBand, TB_BUTTONCOUNT, 0, 0L);

		// Set band info structure
		REBARBANDINFO rbBand;
		rbBand.cbSize = sizeof(REBARBANDINFO);

		rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE | RBBIM_IDEALSIZE;

		if(lpstrTitle != NULL)
			rbBand.fMask |= RBBIM_TEXT;

		rbBand.fStyle = fExtraStyles;
		rbBand.fStyle |= RBBS_CHILDEDGE;

		if(bNewRow)
			rbBand.fStyle |= RBBS_BREAK;

		rbBand.lpText = lpstrTitle;
		rbBand.hwndChild = hWndBand;
		if(nID == 0)	// calc band ID
			nID = ATL_IDW_BAND_FIRST + (int)::SendMessage(hWndReBar, RB_GETBANDCOUNT, 0, 0L);
		rbBand.wID = nID;

		// Calculate the size of the band
		BOOL bRet;
		RECT rcTmp;
		if(nBtnCount > 0)
		{
			bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, nBtnCount - 1, (LPARAM)&rcTmp);
			ATLASSERT(bRet);
			rbBand.cx = (cxWidth != 0) ? cxWidth : rcTmp.right;
			rbBand.cyMinChild = rcTmp.bottom - rcTmp.top;
			if(bFullWidthAlways)
			{
				rbBand.cxMinChild = rbBand.cx;
			}
			else if(lpstrTitle == 0)
			{
				bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, 0, (LPARAM)&rcTmp);
				ATLASSERT(bRet);
				rbBand.cxMinChild = rcTmp.right;
			}
			else
			{
				rbBand.cxMinChild = 0;
			}
		}
		else	// no buttons, either not a toolbar or really has no buttons
		{
			bRet = ::GetWindowRect(hWndBand, &rcTmp);
			ATLASSERT(bRet);
			rbBand.cx = (cxWidth != 0) ? cxWidth : (rcTmp.right - rcTmp.left);
			rbBand.cxMinChild = (bFullWidthAlways) ? rbBand.cx : 0;
			rbBand.cyMinChild = rcTmp.bottom - rcTmp.top;
		}


		rbBand.cxIdeal = rbBand.cx;


		// Add the band
		LRESULT lRes = ::SendMessage(hWndReBar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBand);
		if(lRes == 0)
		{
			ATLTRACE2(atlTraceUI, 0, _T("Failed to add a band to the rebar.\n"));
			return FALSE;
		}

		return TRUE;


	
}


LRESULT CMainFrame::OnRefresh(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	::PostMessage(m_pView->hWnd(), WM_REFRESH,NULL,NULL);
	return 0;
}

LRESULT CMainFrame::OnStop(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	::PostMessage(m_pView->hWnd(),WM_STOP,NULL,NULL);
	return 0;
}

LRESULT CMainFrame::OnSettings(WORD wNotifyCode, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	
	WORD code = 0x10000 - wNotifyCode;
	::PostMessage(m_pView->hWnd(),WM_SETTINGS,code,NULL);
	return 0;
}


void CMainFrame::StatusReport(int panel, tstring status)
{
	m_status.SetText(panel,status.c_str());
}

LRESULT CMainFrame::OnLogging(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	g_logging = !g_logging;
	UISetCheck(ID_LOGGING, g_logging);
		
	return 0;
}


void CMainFrame::SaveSettings(CIniFile &ini)
{
	CWindowWithSettings *pWS = dynamic_cast<CWindowWithSettings *>(m_pView);

	pWS->SaveSettings(ini);

	ini.SetInt(_T("mainframe"),_T("bottom"),m_InitRect.bottom);
	ini.SetInt(_T("mainframe"),_T("top"),m_InitRect.top);
	ini.SetInt(_T("mainframe"),_T("left"),m_InitRect.left);
	ini.SetInt(_T("mainframe"),_T("right"),m_InitRect.right);

	ini.SetInt(_T("mainframe"), _T("maximize"), m_bMaximize);

	ini.SetInt(_T("global"),_T("logging"),g_logging);

	

}



void CMainFrame::LoadSettings(const CIniFile &ini)
{

	CWindowWithSettings *pWS = dynamic_cast<CWindowWithSettings *>(m_pView);
	
	pWS->LoadSettings(ini);

	m_InitRect.bottom	= ini.GetInt(_T("mainframe"),_T("bottom"),600);
	m_InitRect.top		= ini.GetInt(_T("mainframe"),_T("top"),50);
	m_InitRect.left		= ini.GetInt(_T("mainframe"),_T("left"),10);
	m_InitRect.right	= ini.GetInt(_T("mainframe"),_T("right"),1000);

	m_bMaximize			= ini.GetInt(_T("mainframe"), _T("maximize"),FALSE);

	g_logging			= ini.GetInt(_T("global"),_T("logging"),0);
	
	


	if (g_logging)
	{

		TCHAR fname[MAX_PATH];
		::GetModuleFileName(_Module.m_hInst,fname, MAX_PATH);
		PathRemoveExtension(fname);

		tstring fn = fname + tstring(_T(".log"));
		DeleteFile(fn.c_str());
	}

}

LRESULT CMainFrame::OnEditDelete(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& bHandled)
{
	::PostMessage(m_pView->hWnd(),WM_DELETE,NULL,NULL);
	return 0;
}


LRESULT CMainFrame::OnEditRename(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& bHandled)
{
	::PostMessage(m_pView->hWnd(),WM_RENAME,NULL,NULL);
	return 0;
}


LRESULT CMainFrame::OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CAboutDlg dlg;
	dlg.DoModal();
	return 0;
}

LRESULT CMainFrame::OnJunk(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	/*tstring path = get_desktop_path();
	path+=_T("\\testjunk\\");
	for (int i = 0;i<400;i++)
	{
		tstring fname = _T("junk");
		TCHAR b[5];
		fname+=_itot(i,b,10);
		CArch arch;
		arch.OpenWrite(path + fname + _T(".mp3"));
		arch.WriteFile(_T("hello"));
		arch.Close();
	}*/
	return 0;
}


LRESULT CMainFrame::OnUpdateStatusBar(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	int panel = wParam;
	tstring *pStr = reinterpret_cast<tstring *>(lParam);
	tstring safe_string = *pStr;
	delete pStr;

	m_status.SetText(panel, safe_string.c_str());
	return 0;

}


LRESULT CMainFrame::OnHelp(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	tstring link = _T("http://www.guerillasoft.com/Encspot/help.html");
	::ShellExecute(0, _T("open"), link.c_str(), 0, 0, SW_SHOWNORMAL);
	return 0;

}
LRESULT CMainFrame::OnControlAnimation(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{

	if (wParam == 0)
	{
		m_animate.Stop();
		m_animate.Seek(0);
	}
	else
		m_animate.Play(0,-1,-1);
	return 0;
}

LRESULT CMainFrame::OnSize(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{

	if (wParam == SIZE_MAXIMIZED)
		m_bMaximize = TRUE;
	else
	{
		if (m_bMaximize)
		{
			m_bMaximize = FALSE;
			MoveWindow(&m_InitRect);
		}
		else
		{
			int width	= LOWORD(lParam);
			int height	= HIWORD(lParam);
			m_InitRect.right = m_InitRect.left + width;
			m_InitRect.bottom= m_InitRect.top  + height;
		}	
	}

	bHandled = FALSE;

	return 0;
}


LRESULT CMainFrame::OnMoving(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	GetWindowRect(&m_InitRect);	
	bHandled = FALSE;
	return 0;
}



LRESULT CMainFrame::OnSizing(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	GetWindowRect(&m_InitRect);	
	bHandled = FALSE;
	return 0;
}

LRESULT CMainFrame::OnShellAccel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{

	::PostMessage(m_pView->hWnd(), WM_SHELLOP, wID, NULL);

	return 0;
}


